#!/usr/bin/perl -w

# Copyright 2014, 2015, 2016, 2017, 2019, 2021 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.

use 5.010;
use strict;
use FindBin;
use POSIX 'ceil';
use List::Util 'max','min','sum';
use Math::Libm 'hypot';
use Math::PlanePath::CCurve 117;
use Memoize 'memoize';
use Math::PlanePath::Base::Digits 'bit_split_lowtohigh', 'digit_join_lowtohigh';

use FindBin;
use lib::abs "$FindBin::Bin/../tools";
use lib::abs "$FindBin::Bin/../../dragon/tools";
use MyCCurve ':all';

# uncomment this to run the ### lines
# use Smart::Comments;

my @ipc;
our @right_boundary_table;
$|=1;


{
  # turn() mod 4 re-map symbols
  # A092412
  # dir() mod 4 re-map symbols
  # A179868  count 1s mod 4
  # A083661 count 0-bits something

  # A106147     1, 4, 4, 3, 4, 3, 3, 2, 4, 3, 3, 2, 3, 2, 2, 1, 4, 3, 3, 2, 3
  # 1->{2, 1},
  # 2->q*{3, 2}+(1-q}*{2, 3},
  # 3->{4, 3},
  # 4->q*{1, 4}+(1-q)*{4, 1}
  # with q=1

  require Math::Permute::Array;
  require Math::OEIS::Grep;
  my $count = 0;
  foreach my $funcname ('dir','turn') {
    my $func = MyCCurve->can($funcname) or die "oops, no $funcname";
    my @values = map {$func->($_)%4} 3 .. 50;
    my $p = Math::Permute::Array->new([0..3]);
    print "perms ",$p->cardinal,"\n";
    do {
      my $perm = $p->cur;
      foreach my $add (0,1) {
        $count++;
        my $name = "$funcname perm ".join('',@$perm)." plus $add";
        my @new = map {$perm->[$_]+$add} @values;
        Math::OEIS::Grep->search(array => \@new,
                                 name => $name,
                                 verbose => 0,
                                );
      }
    } while ($p->next);
  }
  print "count $count\n";
  exit 0;
}

{
  # turn of ternary count low 0s
  my $k = 10;       # should be even
  my $scale = 2;   # pixels per segment

  require POSIX;
  require File::Spec;
  require File::Temp;
  require Math::Complex;
  my $dir = File::Spec->catdir('/tmp', POSIX::cuserid());
  mkdir $dir;

  my $png_filename  = File::Spec->catfile($dir, 'a007949.png');

  sub ternary_low_0s {
    my ($n) = @_;
    $n > 0 or die;
    my $ret = 0;
    until ($n % 3) {
      $ret++;
      $n /= 3;
    }
    return $ret;
  }

  my @points;
  my $z = Math::Complex->new;
  my $dz = 1;
  my $I = Math::Complex->new(0,1);
  my $n_hi = 3**$k;
  foreach my $n (0 .. $n_hi) {
    push @points, $z;
    $z += $dz;
    my $turn = 1 - ternary_low_0s($n+1);
    $dz *= $I**$turn;
  }
  {
    my $rot = $I ** -int($k/2);
    @points = map {$_ *= $rot} @points;
  }
  @points = map {$scale*$_} @points;
  @points = map {[$_->Re, $_->Im]} @points;

  my @x = map {$_->[0]} @points;
  my @y = map {$_->[1]} @points;
  my $size = max( max(@x)-min(@x), max(@y)-min(@y) );

  my $width = (max(@x)-min(@x));
  my $height = (max(@y)-min(@y));
  $width  = int($width*1.1);
  $height = int($height*1.1);
  print "size $width x $height  points $n_hi\n";

  require Geometry::AffineTransform;
  my $affine = Geometry::AffineTransform->new;
  $affine->translate( POSIX::round(-(max(@x)+min(@x))/2),
                      POSIX::round(-(max(@y)+min(@y))/2) );
  $affine->translate($width/2, $height/2);
  @points = map {[$affine->transform(@$_)]} @points;
  @x = map {$_->[0]} @points;
  @y = map {$_->[1]} @points;
  print "transformed x ",min(@x)," to ",max(@x),
    "  y ",min(@y)," to ",max(@y),"\n";

  require Image::Base::GD;
  my $image = Image::Base::GD->new (-width => $width, -height => $height);
  $image->rectangle(0,0, $width,$height, 'black', 1);

  print "draw to $#x\n";
  foreach my $i (0 .. $#x-1) {
    $image->line($x[$i],  $y[$i],
                 $x[$i+1],$y[$i+1], 'white');
  }
  {
    my ($x,$y) = $affine->transform(0,0);
    $image->ellipse($x-1,$y-1, $x+1,$y+1, 'red', 1);
  }
  $image->save($png_filename);
  system('optipng',$png_filename);
  exit 0;
}


{
  # polygon piece types

  # distinct in part, rotate and mirror
  # not in OEIS: 1,0,1,1,2,2,3,3,5,6,7,10,14,21,27,37,49
  # distinct in part, rotate and mirror, k even
  # not in OEIS: 1,1,2,3,5,7,14,27,49,80

  # distinct whole curve, rotate and mirror
  # not in OEIS: 1,1,1,2,3,3,4,5,6,6,7,10,16,24,31,41


  # my $poly = [ # '0,0,0,1',
  #              # '0,1,0,1',
  #             '0,0,-1,0',
  #             # '1,0,0,1', '1,1,-1,0', '1,1,0,-1'
  #            ];
  # poly_print($poly);
  # $poly = poly_neg_x($poly);
  # poly_print($poly);
  # exit;

  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (
                 0 .. 15,
                 # 0,2,4,6,8,10,12,14,16,18
                ) {
    my %polys;
    my %segment_to_poly;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    # my $n_lo = PartN($k);
    # my $n_hi = PartN($k+1);
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);

      my $key = "$x,$y,$dx,$dy";
      my $poly;
      foreach my $func (\&MyCCurve::xydxdy_reverse,
                        \&MyCCurve::xydxdy_triangle_neighbour_start,
                        \&MyCCurve::xydxdy_triangle_neighbour_end) {
        my ($ox,$oy, $odx,$ody) = $func->($x,$y, $dx,$dy);
        my $other_key = "$ox,$oy,$odx,$ody";
        if (my $other_poly = $segment_to_poly{$other_key}) {
          if ($poly) {
            if ($poly == $other_poly) {
              ### merge: "same poly = other_poly"
            } else {
              ### merge: "other_poly $other_poly size ".scalar(@$other_poly)." to $poly size ".scalar(@$poly)
              push @$poly, @$other_poly;
              foreach my $seg (@$other_poly) { $segment_to_poly{$seg} = $poly; }
              delete $polys{scalar $other_poly};
            }
          } else {
            # put segment into other_poly
            $poly = $other_poly;
            push @$other_poly, $key;
          }
        }
      }
      if (! $poly) {
        # new poly
        $poly = [$key];
        $polys{scalar $poly} = $poly;
      }
      $segment_to_poly{$key} = $poly;
    }
    my @polys = values %polys;
    my $num_poly = scalar(@polys);
    print "k=$k  Trigons $num_poly\n";

    my %distinct_polys;
    @polys = map {poly_normalize($_)} @polys;
    foreach my $poly (@polys) {
      $distinct_polys{poly_str($poly)} = $poly;
    }
    my $num_distinct_poly = scalar(keys %distinct_polys);
    print "  distinct $num_distinct_poly\n";

    # {
    #   print "  sizes  ";
    #   foreach my $poly (values %distinct_polys) {
    #     my $size = scalar(@$poly);
    #     print "$size,";
    #   }
    #   print "\n";
    # }

    my @num_distinct_by_size;
    print "  distinct by size  ";
    foreach my $poly (values %distinct_polys) {
      my $size = scalar(@$poly);
      $num_distinct_by_size[$size]++;
    }
    foreach my $size (1 .. $#num_distinct_by_size) {
      print $num_distinct_by_size[$size]//0,",";
      # if ($num_distinct_by_size[$size]) {
      #   print "$size\[$num_distinct_by_size[$size]], ";
      # }
    }
    print "\n";

    push @values, $num_distinct_poly;

    if (0 && $k==10) {
      foreach my $str (sort keys %distinct_polys) {
        my $poly = $distinct_polys{$str};
        poly_print($poly);
      }
    }
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose => 1);
  exit 0;

  sub poly_print {
    my ($poly) = @_;
    my $size = scalar(@$poly);
    print "$size triangles  ",poly_str($poly),"\n";
    my ($min_x,$min_y, $max_x,$max_y) = poly_bbox($poly);
    require Image::Base::Text;
    my $scale = 9;
    my $width  = $scale*($max_x-$min_x + 2);
    my $height = $scale*($max_x-$min_x + 2);
    my $image = Image::Base::Text->new (-height => $height,
                                        -width  => $width);
    my $ox = ($scale+1)/2;
    my $oy = ($scale+1)/2;
    $image->xy(0 +        (0-$min_x)*$scale + $ox,
               $height-1-((0-$min_y)*$scale + $oy), '.');

    foreach my $str (@$poly) {
      my ($x,$y, $dx,$dy) = split /,/, $str;
      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      my $sx = 0;
      my $sy = 0;
      my $dir4 = dxdy_to_dir4($dx,$dy);
      if ($dir4 == 1) {        $sy++; }
      if ($dir4 == 2) { $sx--; $sy++; }
      if ($dir4 == 3) { $sx--;        }

      my $char = $dir4;

      #  0 1 2 3 4 5 6 7
      #
      #  . @ @ @ @ @ . . @
      #  @   @ @ @   @ @
      #  @ @   @   @ @ @ @
      #  @ @ @   @ @ @ @ @ @
      #  @ @       @ @ @ @
      #  @           @ @
      #  .             .
      foreach my $i (1 .. $scale-2) {
        foreach my $j (0 .. min($i-1, $scale-2-$i)) {
          $image->xy(0 +          ($x-$min_x)*$scale + $i*$dx + $j*$rx + $sx + $ox,
                     $height-1 - (($y-$min_y)*$scale + $i*$dy + $j*$ry + $sy + $oy),
                     $char);
        }
      }
    }
    $image->save('/dev/stdout');  # FIXME: use print
  }
  sub poly_str {
    my ($poly) = @_;
    return join(' ',@$poly);
  }
  sub poly_normalize {
    my ($poly) = @_;
    my ($min_poly, $min_str);
    foreach (1 .. 2) {
      foreach (1 .. 4) {
        $poly = poly_xy_rotate_plus90($poly);
        my ($min_x,$min_y) = poly_bbox($poly);
        $poly = poly_add_xy($poly, -$min_x,-$min_y);
        $poly = [ sort @$poly ];
        my $str = poly_str($poly);
        if (! defined $min_poly || $str lt $min_str) {
          $min_poly = $poly;
          $min_str = $str;
        }
      }
      $poly = poly_neg_x($poly);
    }
    return $min_poly;
  }
  sub poly_neg_x {
    my ($poly) = @_;
    return [ map {
      my ($x,$y, $dx,$dy) = split /,/;
      $x = -$x;
      if ($dx) {
        $x -= $dx;
      } else {
        $y += $dy;
        $dy = -$dy;
      }
      "$x,$y,$dx,$dy" } @$poly ];
  }
  sub poly_add_xy {
    my ($poly, $add_x,$add_y) = @_;
    return [ map {
      my ($x,$y, $dx,$dy) = split /,/;
      $x += $add_x; $y += $add_y;
      "$x,$y,$dx,$dy" } @$poly ];
  }
  sub poly_xy_rotate_plus90 {
    my ($poly) = @_;
    return [ map {
      my ($x,$y, $dx,$dy) = split /,/;
      ($x,$y) = xy_rotate_plus90($x,$y);
      ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      "$x,$y,$dx,$dy" } @$poly ];
  }
  sub poly_bbox {
    my ($poly) = @_;
    my ($min_x, $min_y, $max_x, $max_y);
    foreach my $str (@$poly) {
      my ($x,$y, $dx,$dy) = split /,/, $str;
      $min_x = min($x,$x+$dx, (defined $min_x ? $min_x : ()));
      $min_y = min($y,$y+$dy, (defined $min_y ? $min_y : ()));
      $max_x = max($x,$x+$dx, (defined $max_x ? $max_x : ()));
      $max_y = max($y,$y+$dy, (defined $max_y ? $max_y : ()));
    }
    return ($min_x,$min_y, $max_x,$max_y);
  }
}

{
  # Lpred cycles

  my $f = NonLpred_k_flat()->clone;
  $f->set_accepting(9,14, 40,41,42,43,44, 38,39);
  $f = $f->MyFLAT::eventually_accepting;
  $f = $f->MyFLAT::separate_sinks;
  $f->MyFLAT::view;
  exit 0;
}
{
  # Lpred eventually

  # 17,21
  # 26,29
  # 32,33, 36,37
  # 45,46,47,48 = d

  my $f = NonLpred_k_flat();
  my @states = $f->MyFLAT::get_eventually_accepting();
  @states = sort {$a<=>$b} @states;
  print "eventually accepting ",join(', ',@states),"\n";

  $f = $f->MyFLAT::eventually_accepting;
  $f = $f->MyFLAT::separate_sinks;
  $f->MyFLAT::view;
  exit 0;
}
{
  # Lpred picture

  my $f = NonLpred_k_flat();
  $f->MyFLAT::FLAT_print_tikz();
  exit 0;
}

{
  # Lpred flat

  my %seen;
  foreach my $state (0 .. $#$Lpred_state_table_aref) {
    my $aref = $Lpred_state_table_aref->[$state];
    my $key = join(',',@$aref);
    if ($seen{$key}) {
      print "$state seen $key previous $seen{$key}\n";
    } else {
      $seen{$key} = $state;
    }
  }

  print scalar(@$Lpred_state_table_aref)," states\n";
  my $f = NonLpred_k_flat();
  print $f->num_states," states\n";
  $f->MyFLAT::view;

  print "prune\n";
  $f->prune;
  print $f->num_states," states\n";

  $f = $f->MyFLAT::minimize (verbose => 1);
  print $f->num_states," states\n";
  # $f->MyFLAT::view;

  # my $orig = NonLpred_k_flat();
  # foreach my $state (sort {$a<=>$b} $f->get_states) {
  #   foreach my $bit (0,1) {
  #     my ($orig_to) = $orig->successors($state,$bit);
  #     my ($min_to)  = $f->successors($state,$bit);
  #     if ($orig_to != $min_to) {
  #       print "$state bit=$bit to $orig_to orig $min_to\n";
  #     }
  #   }
  # }

  exit 0;
}


{
  my $f = Rpred_k_flat()
    ->MyFLAT::reverse
    ->MyFLAT::minimize;
  print $f->{'name'},"  ",$f->num_states," states\n";
  $f->MyFLAT::view;
  exit 0;
}
{
  # how many crossings
  #
  #      ^
  #      |
  # ----> ---->
  #      ^
  #      |
  #
  # with nothing else at the same point
  #
  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my $xydxdy_to_n = sub {
      my ($x,$y,$dx,$dy) = @_;
      my $n = $path->xyxy_to_n($x,$y, $x+$dx, $y+$dy);
      return (defined $n && $n <= $n_hi ? $n : undef);
    };
    my $n_is_crossing = sub {
      my ($n) = @_;
      $n >= 1 || return 0;
      my $t = turn($n) % 4;
      ($t == 0 || $t == 2) or return 0;  # turn straight
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my @other = (0);
      my $other_n;
      foreach my $dir (1 .. 3) {
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
        my $n2 = $xydxdy_to_n->($x,$y, $dx,$dy);
        $other[$dir] = defined $n2;
        if (defined $n2) { $other_n = $n2; }
      }
      sum(@other) == 1 or return 0;
      return ($other[1] || $other[3]);
    };

    my $count = 0;
    foreach my $n (1 .. $n_hi-1) {
      my $cross = $n_is_crossing->($n);
      $count += $cross;

      # if ($k<=10 && $cross) {
      #   my ($x,$y) = $path->n_to_xy($n);
      #   printf "n=$n  %*b at $x,$y\n", $k,$n;
      # }
    }
    $count /= 2;
    # print "k=$k  $count\n";
    print "$count,\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # how many turn touches
  #
  #       ^
  #       |
  #      /
  # ---->  /<----
  #       |
  #       v
  #
  # with nothing else at the same point
  #
  # turn left, no other segments
  # recurrence_guess([0, 0, 0, 0, 0, 0, 0, 0, 2, 4, 10, 24, 56, 108, 228, 456, 926])
  # = QI2o
  #
  # turn right, no other segments
  # recurrence_guess([0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 2, 4, 10, 24, 56, 108, 228])
  # = 2 expansions later

  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);

    my $xydxdy_to_n = sub {
      my ($x,$y,$dx,$dy) = @_;
      my $n = $path->xyxy_to_n($x,$y, $x+$dx, $y+$dy);
      return (defined $n && $n <= $n_hi ? $n : undef);
    };
    my $n_is_turn_touch = sub {
      my ($n) = @_;
      $n >= 1 || return 0;
      (turn($n)%4) == 3 || return 0;
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $n2 = $xydxdy_to_n->($x,$y, -$dx,-$dy) // return 0;  # opposite leave
      return 0 if $n2 < $n;

      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      return 0 if defined $xydxdy_to_n->($x,$y, $rx,$ry);
      return 0 if defined $xydxdy_to_n->($x,$y, -$rx,-$ry);
      return 1;
    };

    my $count = 0;
    my @count_by_lows;
    foreach my $n (1 .. $n_hi-1) {
      my $touch = $n_is_turn_touch->($n);
      $count += $touch;
      $count_by_lows[CountLowZeros($n)] += $touch;

      # if ($k<=10 && $touch) {
      #   my ($x,$y) = $path->n_to_xy($n);
      #   printf "n=$n  %*b at $x,$y\n", $k,$n;
      # }
    }
    @count_by_lows = map {$_//0} @count_by_lows;
    # print "k=$k  $count   ",join(',',@count_by_lows),"\n";
    print "$count,\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  our $Lpred_mask_segs = MySegsConf->new
    (name => 'surround inner',
     side => 0,
     segs => [
              "0,0,0", "1,0,2",      "1,0,0", "2,0,2",
              "0,0,1", "0,1,3",      "1,0,1", "1,1,3",
              "0,0,2", "-1,0,0",     
              "0,0,3", "0,-1,1",     "1,0,3", "1,-1,1",
              "-1,-1,0", "0,-1,0", "1,-1,0",
             ]);
  # $Lpred_mask_segs->print;
  my %side_flip = (L => 'R', R => 'L');

  my @state_to_conf = (MySegsConf->new);  # 0 = empty
  my %conf_to_state;
  my $conf_to_state = sub {
    my ($conf) = @_;
    my $key = $conf->key;
    my $state = $conf_to_state{$key};
    if (! defined $state) {
      push @state_to_conf, $conf;
      $state = $#state_to_conf;
      $conf_to_state{$key} = $state;
      $conf->set_name("state $state  ");
      # print "new state $state\n";
      # $state_to_conf[$state]->print;
    }
    return $state;
  };
  $conf_to_state->(MySegsConf->initial(within => $Lpred_mask_segs,
                                       side => 'L'));
  my @pending = (1);
  while (@pending) {
    my $state = shift @pending;
    my $from_conf = $state_to_conf[$state];
    print "----------------------------\nstate $state\n";
    $from_conf->print;
    foreach my $bit (0,1) {
      print "bit=$bit\n";
      my $to_conf = $from_conf->expand($bit);

      if ($from_conf->contains('1,0,2')) {
        $to_conf->{'side'} = $side_flip{$to_conf->{'side'}};
      }

      $to_conf->print;
      my $to_state = $conf_to_state->($to_conf);
      if ($to_state > $state) { push @pending, $to_state; }
    }
  }
  exit 0;

  {
    # Segments are represented by strings "X,Y,DIR" where X,Y are integer
    # coordinates, and DIR = 0 to 3.  DIR is direction along the curve and
    # therefore direction expansion, so on expansion get new segments on the
    # right always.

    package MySegsConf;
    use Carp 'croak';
    use List::Util 'min','max';
    use List::MoreUtils;
    use MyCCurve '@dir4_to_dx', '@dir4_to_dy';

    sub new {
      my $class = shift;   # ($class, segs=>$s, side=>$side)
      if (ref $class) { $class = ref $class; }
      my $self = bless { segs => [], @_ }, $class;
      $self->{'segshash'} = { map {$_=>1} $self->segs_list };
      foreach my $seg ($self->segs_list) {
        $self->seg_validate($seg);
      }
      ### $self
      return $self;
    }
    sub seg_validate {
      my ($self, $seg) = @_;
      unless ($seg =~ /^(-?[1-9][0-9]*|0),(-?[1-9][0-9]*|0),[0123]$/) {
        croak "bad seg ",$seg;
      }
    }
    sub key {
      my ($self) = @_;
      return ($self->{'key'} //= join(' ',
                                      "side=$self->{side}",
                                      sort $self->segs_list));
    }
    sub initial {
      my $class = shift;
      return $class->new (side => 0, segs => ['0,0,0'], @_);
    }
    sub equals {
      my ($self, $other) = @_;
      return $self->key eq $other->key;
    }
    sub contains {
      my ($self, $seg) = @_;
      return $self->{'segshash'}->{$seg};
    }
    sub is_subset {
      my ($self, $superset) = @_;
      foreach my $seg ($self->segs_list) {
        if (! $superset->contains($seg)) { return 0; }
      }
      return 1;
    }
    sub segs_list {
      my ($self) = @_;
      return @{$self->{'segs'}};
    }
    sub num_segs {
      my ($self) = @_;
      return scalar @{$self->{'segs'}};
    }

    # return a new MySegsConf with each point $x,$y moved to $x+$dx, $y+$dy
    sub translate {
      my ($self, $dx,$dy) = @_;
      if ($dx==0 && $dy==0) { return $self; }
      my $name = $self->{'name'};
      if (defined $name) {
        $name = "$name moved $dx,$dy";
      }
      return $self = $self->new
        (name => $name,
         side => $self->{'side'},
         segs => [ map { my ($x,$y,$dir) = split /,/;
                         $x += $dx;
                         $y += $dy;
                         "$x,$y,$dir"
                       } $self->segs_list ]);
    }

    # return a new MySegsConf with each segment rotated by $ddir, so each
    # $dir becomes $dir+$ddir, mod 4
    sub rotate {
      my ($self, $ddir) = @_;
      $ddir %= 4;
      if ($ddir == 0) { return $self; }

      my $name = $self->{'name'};
      if (defined $name) {
        $name = "$name rotate ".($ddir==1   ? "+90"
                                 : $ddir==2 ? "180"
                                 : $ddir==3 ? "-90"
                                 : $ddir);
      }
      return $self = $self->new
        (name => $name,
         side => $self->{'side'},
         segs => [ map { my ($x,$y,$dir) = split /,/;
                         foreach (1 .. $ddir) {
                           ($x,$y) = MyCCurve::xy_rotate_plus90($x,$y)
                         }
                         $dir = ($dir + $ddir) % 4;
                         "$x,$y,$dir"
                       } $self->segs_list ]);
    }

    # return a new MySegsConf with the segment common to all the given confs
    sub intersection {
      my $self = shift;
      foreach my $other (@_) {
        $self = $self->new
          (side => $self->{'side'},
           within => $self->{'within'},
           segs => [ grep {$other->contains($_)} $self->segs_list ]);
      }
      return $self;
    }
    sub difference {
      my $self = shift;
      foreach my $other (@_) {
        $self =$self->new
          (side   => $self->{'side'},
           within => $self->{'within'},
           segs => [ grep {! $other->contains($_)} $self->segs_list ]);
      }
      return $self;
    }
    sub union {
      my $self = shift;
      my $name = $self->{'name'};
      if (defined $name) { $name = "$name union"; }
      return $self->new (name   => $name,
                         side   => $self->{'side'},
                         within => $self->{'within'},
                         segs   => [ List::MoreUtils::uniq
                                     (map {$_->segs_list} $self,@_) ]);

    }

    sub set_name {
      my ($self, $name) = @_;
      $self->{'name'} = $name;
      return $self;
    }
    sub set_within {
      my ($self, $within) = @_;
      $self->{'within'} = $within;
      return $self;
    }

    sub expand {
      my ($self, $bit) = @_;
      ($bit==0 || $bit==1) or die;
      if (! defined $self->{'side'}) {
        croak 'No side for conf expand';
      }

      #  0 ------> 1
      #   0\     ^
      #     \   /
      #     1v /0

      my @segs = map { my ($x,$y,$dir) = split /,/;
                       ($x,$y) = MyCCurve::mul($x,$y);
                       my $x2 = $x + $dir4_to_dx[$dir];
                       my $y2 = $y + $dir4_to_dy[$dir];
                       my $dir2 = ($dir+1) % 4;
                       ("$x,$y,$dir", "$x2,$y2,$dir2")
                     } $self->segs_list;
      if ($bit) {
        @segs = map { my ($x,$y,$dir) = split /,/;
                      $x--;
                      ($x,$y) = MyCCurve::xy_rotate_minus90($x,$y);
                      $dir = ($dir-1) % 4;
                      "$x,$y,$dir"
                    } @segs;
      }
      my $expand = $self->new (side => $self->{'side'}, segs => \@segs);

      if (defined (my $within = $self->{'within'})) {
        $expand = $expand->intersection($within);
        $expand->{'within'} = $within;
      }
      if (defined (my $name = $self->{'name'})) {
        unless ($name =~ /[0-9]$/) { $name .= ' '; }
        $expand->{'name'} = $name . '0';
      }
      return $expand;
    }

    # bounding box around the segments
    sub bbox {
      my ($self) = @_;
      my $x_min=0;
      my $x_max=1;
      my $y_min=0;
      my $y_max=0;
      foreach my $seg ($self->segs_list) {
        my ($x,$y,$dir) = split /,/, $seg;
        foreach (0,1) {
          $x_min = min($x_min, $x); $x_max = max($x_max, $x);
          $y_min = min($y_min, $y); $y_max = max($y_max, $y);
          $x += $dir4_to_dx[$dir];
          $y += $dir4_to_dy[$dir];
        }
      }
      return ($x_min,$y_min, $x_max,$y_max);
    }
    sub print {
      my ($self) = @_;
      my $segshash = $self->{'segshash'};
      print "conf ",
        $self->{'name'} // '',
        " side=", $self->{'side'}//'undef', "\n";
      print "   ",join(' ',$self->segs_list),"\n";

      my ($x_min,$y_min, $x_max,$y_max) = $self->bbox;
      ### $x_min
      ### $x_max
      ### $y_min
      ### $y_max

      foreach my $y (reverse $y_min .. $y_max) {
        my $yp1 = $y+1;
        if ($y<$y_max) {
          # above verticals
          print "   ";
          foreach my $x ($x_min .. $x_max) {
            if ($self->contains("$x,$y,1"))   { print "^"; }
            else                              { print " "; }
            if ($self->contains("$x,$yp1,3")) { print "|    "; }
            else                              { print "     "; }
          }
          print "\n";
          print "   ";
          foreach my $x ($x_min .. $x_max) {
            if ($self->contains("$x,$y,1"))   { print "|"; }
            else                              { print " "; }
            if ($self->contains("$x,$yp1,3")) { print "|    "; }
            else                              { print "     "; }
          }
          print "\n";
          print "   ";
          foreach my $x ($x_min .. $x_max) {
            if ($self->contains("$x,$y,1"))   { print "|"; }
            else                              { print " "; }
            if ($self->contains("$x,$yp1,3")) { print "v    "; }
            else                              { print "     "; }
          }
          print "\n";
        }

        # row
        print "   ";
        foreach my $x ($x_min .. $x_max) {
          my $xp1 = $x+1;
          print $x==0&&$y==0 ? "O" : "*";
          if ($self->contains("$x,$y,0"))      { print $x==0&&$y==0
                                                   ? "====>" : "---->"; }
          else                                 { print "     "; }
        }
        print "\n";
        print "   ";
        foreach my $x ($x_min .. $x_max) {
          my $xp1 = $x+1;
          print $x==0&&$y==0 ? "O" : "*";
          if ($self->contains("$xp1,$y,2"))    { print "---->"; }
          else                                 { print "     "; }
        }
        print "\n";
      }
      print "\n";
    }
  }

}



{
  # segments in direction up to N

  my $path = Math::PlanePath::CCurve->new;
  my $formula = sub {
    my ($n,$d) = @_;
    my ($x,$y) = $path->n_to_xy($n);
    foreach (1 .. $d) { ($x,$y) = xy_rotate_plus90($x,$y); }
    return $n-$x;
  };

  my (@v0,@v1,@v2,@v3);
  my $rot = 0;
  my @counts = (0,0,0,0);
  foreach my $n (0 .. 30) {
    push @v0, $counts[0];
    push @v1, $counts[1];
    push @v2, $counts[2];
    push @v3, $counts[3];

    my @formulas = map {$formula->($n,$_)} 0 .. 3;
    print "n=$n  ",join(',',@counts),"  ",join(',',@formulas),"\n";
    my $dir = dir($n);
    $counts[$dir%4]++;
  }

  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@v0, name=>'v0');
  Math::OEIS::Grep->search(array => \@v1, name=>'v1');
  Math::OEIS::Grep->search(array => \@v2, name=>'v2');
  Math::OEIS::Grep->search(array => \@v3, name=>'v3');
  exit 0;
}

{
  require Image::Base::GD;
  my $width = 800;
  my $height = 600;
  my $scale = 4;
  my $image = Image::Base::GD->new (-height => $height,
                                    -width  => $width);
  $image->rectangle(0,0, $width-1,$height-1, 'black');
  my $colour = 'white';
  my $tmax = 0;
  my $tmin = 0;
  foreach my $x (0 .. $width-1) {
    foreach my $y (0 .. $height) {
      next if $x==0 && $y==0;
      my $t = turn_at_xy($x,$y) % 4;
      # my $t = 1-turn_at_xy($x,$y) + 1;
      $tmax = max($tmax, $t);
      $tmin = min($tmin, $t);
      my $c = $t / 6 * 255;
      $c = max($c, 0);
      $c = min($c, 255);
      my $colour = sprintf '#%02X%02X%02X', $c, $c, $c;
      $image->xy($x,$y, $colour);
    }
  }
  print "t range $tmin, $tmax\n";
  $image->save('/tmp/cc.png');
  require IPC::Run;
  push @ipc, IPC::Run::start(['xzgv','/tmp/cc.png'],'&');
  exit 0;
}

{
  # turn at x,y - goes as z mod b^4 maybe
  #

  # ..0.......0.......0.......0.......0.......0.......0.......0..
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # ..............0...............................0..............
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # ..0.......0.......0.......0.......0.......0.......0.......0..
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # ..0.......0.......0.......0.......0.......0.......0.......0..
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # .............................................................
  # .0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.0.
  # ..............................+..............................

  # .111.1.1.111.1.1.111.1.1.111.111.111.1.1.111.1.1.111.1.1.111.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.111.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.
  # 1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1.1
  # .111.1.1.111.111.111.1.1.111.1+1.111.1.1.111.111.111.1.1.111.

  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2
  # .............................................................
  # ......2...............2...............2...............2......
  # .............................................................
  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2
  # .............................................................
  # .............................................................
  # .............................................................
  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2
  # .............................................................
  # ..............................+..............................
  # .............................................................
  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2
  # .............................................................
  # .............................................................
  # .............................................................
  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2
  # .............................................................
  # ......2...............2...............2...............2......
  # .............................................................
  # 2...2...2...2...2...2...2...2...2...2...2...2...2...2...2...2

  # 3...3...3...3.3.3...3...3...3.3.3...3...3...3.3.3...3...3...3
  # .............................................................
  # ..3...3...3...3...3...3...3...3...3...3...3...3...3...3...3..
  # .............................................................
  # 3...3...3...3...3...3...3...3...3...3...3...3...3...3...3...3
  # .............................................................
  # ..3...3...3...3...3...3...3...3...3...3...3...3...3...3...3..
  # .............................................................
  # 3...3.3.3...3...3...3.3.3...3.+.3...3.3.3...3...3...3.3.3...3
  # .............................................................
  # ..3...3...3...3...3...3...3...3...3...3...3...3...3...3...3..
  # .............................................................
  # 3...3...3...3...3...3...3...3...3...3...3...3...3...3...3...3
  # .............................................................
  # ..3...3...3...3...3...3...3...3...3...3...3...3...3...3...3..
  # .............................................................
  # 3...3...3...3.3.3...3...3...3.3.3...3...3...3.3.3...3...3...3

  my $path = Math::PlanePath::CCurve->new;
  my %turn = ('0,0' => '+');
  my %count;
  my $k = 14;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($ex, $ey) = $path->n_to_xy($n_hi);
  my @offset_x = (0, $ex, -$ey,  $ey, -$ex, $ex+$ey, $ex-$ey, $ex+$ey, $ex-$ey);
  my @offset_y = (0, $ey,  $ex, -$ex, -$ey, $ex-$ey, $ex+$ey, $ex+$ey, $ex-$ey);
  foreach my $o (0 .. $#offset_x) {
    foreach my $rot (0 .. 3) {
      foreach my $n (1 .. $n_hi-1) {
        my ($x,$y) = $path->n_to_xy($n);
        foreach (1 .. $rot) { ($x,$y) = xy_rotate_plus90($x,$y); }
        $x += $offset_x[$o];
        $y += $offset_y[$o];
        my $t = turn($n) % 4;
        my $xy = "$x,$y";
        if (defined $turn{$xy}) {
          my $diff = $turn{$xy} == $t ? "" : "diff *****";
          if ($diff) {  #  || $count{$xy}>=3
            print "N=$n  $x,$y  $diff $count{$xy}\n";
          }
        }
        $turn{$xy} = $t;
        $count{$xy}++;
      }
    }
  }
  my %m = (0 => '.',
           1 => '1',
           2 => '.',
           3 => '.',
           ' ' => ' ',
           '+' => '+');
  foreach my $y (reverse -20 .. 20) {
    foreach my $x (-30 .. 30) {
      my $t = $turn{"$x,$y"} // ' ';
      $t = $m{$t} // die "unknown $t";
      print $t;
    }
    print "\n";
  }
  exit 0;
}

{
  # boundary segments in direction
  # (not in OEIS)
  # even levels, rotate
  #   v3 2*A011377 Expansion of 1/((1-x)*(1-2*x)*(1-x^2))
  #      2*A178420 Partial sums of floor(2^n/3)
  # odd levels
  #   total A131064 = 5*2^n-4n-4
  #      at n+1   10*s^n-4n-8
  # odd levels, unrotate
  #   v0=v1 A079583 3*2^n - n - 2
  #   v2=v3 A000295 Eulerian numbers
  #      2*2^n -3*n -2
  #      10*2^n-4*n-8 - 2*(3*2^n-n-2)
  #      = 4*2^n -2*n - 4
  #     2*2^n - n - 4
  #
  # v0 =            Rpart[0] + Rpart[1]   + Rpart[7]  + Rpart[8]  + Rpart[9] ...
  #
  # v1 = Rpart[1] + Rpart[2] + Rpart[3]   + Rpart[10] + Rpart[11] + ...
  #
  # v2 = Rpart[3] + Rpart[4] + Rpart[5]   + Rpart[11] + Rpart[12] + ...
  #
  # v3 = Rpart[5] + Rpart[6] + Rpart[7]   + Rpart[13] + Rpart[14] + ...
  #
  #    + Rpart[2] + Rpart[10] + ...
  #    + Rpart[3] + Rpart[11] + ...
  #
  # GP-DEFINE  read("c-curve-defines.gp");
  # GP-DEFINE  SBstart(k,d) = {
  # GP-DEFINE    my(debug=0);
  # GP-DEFINE    if(debug,print("SBstart("k","d")"));
  # GP-DEFINE    my(j=-1 + 2*(d%4), total=0);
  # GP-DEFINE    while(j<k,
  # GP-DEFINE       if(j>=0, total+=Rpart(j)/2;   \\ -1
  # GP-DEFINE                if(debug,print("  j="j" add "Rpart(j)/2)));
  # GP-DEFINE       j++; if(j>=k,break());
  # GP-DEFINE       total+=Rpart(j);              \\ 0
  # GP-DEFINE       if(debug,print("  j="j" add "Rpart(j)));
  # GP-DEFINE       j++; if(j>=k,break());
  # GP-DEFINE       total+=Rpart(j)/2;            \\ +1
  # GP-DEFINE       if(debug,print("  j="j" add "Rpart(j)/2));
  # GP-DEFINE       j+=6; if(j>=k,break()));
  # GP-DEFINE    total;
  # GP-DEFINE  }
  # GP-DEFINE  SB(k,d) = SBstart(k+1,d) + SBstart(k,k-d);
  # GP-DEFINE  SBstart_list(k) = vector(4,d,d--; SBstart(k,d));
  # GP-DEFINE  SB_list(k) = vector(4,d,d--; SB(k,d));
  # GP-DEFINE  SB_total(k) = vecsum(SB_list(k));
  # GP-Test  vector(100,k,k--; SB_total(k)) == vector(100,k,k--; R(k))
  # vector(10,k,k--; SBstart(k,0))
  # vector(10,k,k--; SB(k,0))
  # vector(5,k,k--; SBstart_list(k))
  # for(k=0,12, print(SB_list(k)));
  # 
  # vector(10,k,k--; [dir(0),dir(2^k-1)])
  #
  #      *
  #      ^
  #      |
  # *--->*
  # GP-Test  SB_list(1) == [1,1,0,0]
  #
  # *<---*
  #      ^
  #      |
  #      *
  #      ^
  #      |
  # *--->*
  # GP-Test  SB_list(2) == [1,2,1,0]
  # GP-Test  SBstart_list(3) == [1,2,0,0]
  #
  #    *<---*<---*
  #    |         ^
  #    v         |
  #    *         *<---*
  #                   ^
  #                   |
  #                   *
  #                   ^
  #                   |
  #              *--->*       
  # GP-Test  SB_list(3) == [1,3,3,1]
  # GP-Test  SBstart_list(4) == [1,3,1,0]
  unshift @INC, "$FindBin::Bin/../../path/xt";
  require MyOEIS;
  require Math::OEIS::Grep;
  my $path = Math::PlanePath::CCurve->new;
  my (@v0,@v1,@v2,@v3,@total);
  my $rot = 0;
  for (my $level = 1; $level < 16; $level+=1) {
    my $n_start = $path->n_start;
    (my $n_lo, my $n_hi) = $path->level_to_n_range($level);

    my $points = MyOEIS::path_boundary_points ($path, $n_hi,
                                               side => 'right');
    my @count = (0,0,0,0);
    foreach my $i (1 .. $#$points) {
      my $p1 = $points->[$i-1];
      my $p2 = $points->[$i];
      my ($x1,$y1) = @$p1;
      my ($x2,$y2) = @$p2;
      my $dx = $x2-$x1;
      my $dy = $y2-$y1;
      my $dir = dxdy_to_dir4($dx,$dy);
      $count[$dir]++;
    }
    # foreach (1 .. $rot) {  # rotate
    #   ($count[0],$count[1],$count[2],$count[3])
    #     = ($count[3],$count[0],$count[1],$count[2]);
    # }
    foreach (1 .. $rot) {    # unrotate
      ($count[0],$count[1],$count[2],$count[3])
        = ($count[1],$count[2],$count[3],$count[0]);
    }
    print "level=$level  $count[0] $count[1] $count[2] $count[3]\n";

    push @v0, $count[0];
    push @v1, $count[1];
    push @v2, $count[2];
    push @v3, $count[3];
    push @total, sum(@count);
    # $rot++;
  }

  Math::OEIS::Grep->search(array => \@v0, name=>'v0');
  Math::OEIS::Grep->search(array => \@v1, name=>'v1');
  Math::OEIS::Grep->search(array => \@v2, name=>'v2');
  Math::OEIS::Grep->search(array => \@v3, name=>'v3');
  Math::OEIS::Grep->search(array => \@total, name=>'total');
  exit 0;
}

{
  # how many straight ahead lines in both directions
  #    x,y
  #     ----->---->
  #     <----<-----
  #
  # pattern      0,0,0,0,0,0,0,0,2,6,18,52,140,360,872,2080,4798,10994,24678,55164,121920
  # consecutives 0,0,0,0,0,0,0,0,2,6,16,44,116,276,640,1456,3254,7146,15536,33500,71784
  #
  conf_create_all();
  # print "total configurations ", scalar(@index_to_conf), "\n";
  foreach my $conf (conf_make(1,2, 12,13),    # horizontal right
                    conf_make(1,2, 6,5),      # horizontal left
                    conf_make(7,8, 4,3),      # vertical left
                    conf_make(10,11, 15,14),  # vertical right
                   ) {
    # index: $conf_to_index{$conf}
  }
  # {
  #   my $want_conf = conf_make(1,2, 6,5);      # horizontal left
  #   my $count = 0;
  #   foreach my $conf (@index_to_conf) {
  #     if (($conf & $want_conf) == $want_conf) {
  #       print conf_string($conf),"\n";
  #       $count++;
  #     }
  #   }
  #   print "count $count pattern and surrounding\n";
  # }

  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 26) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    my $count_pattern = 0;
    my $count_consecutive = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      my ($x3,$y3) = ($x2+$dx, $y2+$dy);

      my $n2 = $path->xyxy_to_n($x2,$y2, $x3,$y3) // next;  # seg after
      next if $n2 >= $n_hi;
      my $n3 = $path->xyxy_to_n($x3,$y3, $x2,$y2) // next;  # and back
      next if $n3 >= $n_hi;
      next if $n3 < $n;
      my $n4 = $path->xyxy_to_n($x2,$y2, $x,$y) // next;    # and reversal
      next if $n4 >= $n_hi;

      if ($k == 9) {
        print "at n=$n xy=$x,$y  cf end $n_hi $x_hi,$y_hi\n";
      }
      $count_pattern++;
      if ($n + 1 == $n2  && $n4 == $n3 + 1) {
        $count_consecutive++;
      }
    }
    print "k=$k  $count_pattern  $count_consecutive\n";
    push @values, $count_pattern;
  }

  require Math::OEIS::Grep;
  print join(',',@values),"\n";
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}
{
  # other n state transitions

  require MyFSM;
  my %table;
  my %accepting = (same => 1);
  my @pending = ([0,1, 0,0, 0],
                 [0,2, 0,0, 0],
                 [0,3, 0,0, 0],);
  my $state_to_name = sub {
    my ($state) = @_;
    my ($old_dir,$new_dir, $delta_x,$delta_y) = @$state;
    my $out = ''; # ($bit ^ ($delta_x+$delta_y)) & 1;
    my $flip = ''; # ($bit == $out ? "same" : "flip");
    return "$old_dir,$new_dir $delta_x,$delta_y $flip$out";
  };
  my $initial = $state_to_name->($pending[0]);
  my %seen;
  my %allow_delta = ('0,0' => 1,

                     '1,0' => 1,
                     '-1,0' => 1,
                     '0,1' => 1,
                     '0,-1' => 1,

                     '1,1' => 1,
                     '-1,1' => 1,
                     '1,-1' => 1,
                     '-1,-1' => 1,

                     '0,2' => 1,
                     '0,-2' => 1,
                     '2,0' => 1,
                     '-2,0' => 1,

                     '1,2' => 1,
                     '-1,2' => 1,
                     '1,-2' => 1,
                     '-1,-2' => 1,
                     '2,1' => 1,
                     '2,-1' => 1,
                     '-2,1' => 1,
                     '-2,-1' => 1,
                    );
  require Math::Complex;
  my $b = Math::Complex->new(1,1);
  my @dir4_to_z = (Math::Complex->new(1,0),
                   Math::Complex->new(0,1),
                   Math::Complex->new(-1,0),
                   Math::Complex->new(0,-1));
  while (@pending) {
    my $state = shift @pending;
    my ($old_dir,$new_dir, $delta_x,$delta_y, $bit) = @$state;
    my $from = $state_to_name->($state);
    next if $seen{$from}++;

    print "$from\n";
    if ($old_dir == $new_dir && $delta_x == 0 && $delta_y == 0) {
      print "  is accepting\n";
      $accepting{$from} = "same";
    }

    foreach my $bit (0,1) {
      ($old_dir,$new_dir, $delta_x,$delta_y) = @$state;
      my $delta = Math::Complex->new($delta_x,$delta_y);
      if ($bit) {
        $old_dir = ($old_dir - 1) % 4;
      }

      my $s = $dir4_to_z[$old_dir];
      my $z = $bit*$s + $delta;
      my $new_bit = ($z->Re + $z->Im) % 2;
      if ($new_bit) {
        $new_dir = ($new_dir - 1) % 4;
      }
      my $h = $dir4_to_z[$new_dir];
      $delta += $bit*$s - $new_bit*$h;
      $delta /= $b;
      $delta_x = $delta->Re;
      $delta_y = $delta->Im;

      my $new_state = [$old_dir,$new_dir, $delta_x,$delta_y];
      my $to = $state_to_name->($new_state);
      $table{$from}->{$bit} = $to;
      my $flip = ($bit == $new_bit ? "same" : "flip");
      print "  $bit -> $to  bits old=$bit new=$new_bit $flip  by z=$z to delta=$delta\n";

      # if (($delta->Re + $delta->Im) % 2) {
      #   print "delta = $delta not even\n";
      #   die;
      # }
      if(! $allow_delta{"$delta_x,$delta_y"}) {
        print "bit   $bit  unexpected delta $delta_x,$delta_y\n";
        print "from  $from\n";
        print "to    $to\n";
        die;
      }
      push @pending, $new_state;
    }
  }

  my $fsm = MyFSM->new(table => \%table,
                       initial => $initial,
                       accepting => \%accepting,
                       title => 'N Other Transitions',
                       flow => 'east',
                      );

  # my @accepting = keys %accepting;
  # foreach my $i (1 .. $#accepting) {
  #   $fsm->merge_state($accepting[0],$accepting[1]);
  # }

  # $fsm = $fsm->digit_delta;
  $fsm->simplify (verbose => 1);
   $fsm = $fsm->reverse;
  $fsm->simplify (verbose => 1);
  $fsm->renumber_states;
  # $fsm->print_tikz;
  $fsm->view;
  exit 0;
}

#------------------------------------------------------------------------

{
  # where is QI2o first 2-inward-opposite square

  #    x3,y3  -----> x4,y4
  #      ^             |
  #      |             v
  #    x2,y2  <------ x,y

  my $path = Math::PlanePath::CCurve->new;
  my $k=7;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  print "k=$k  end $x_hi,$y_hi\n";

  my $xyxy_to_n = sub {
    my ($x1,$y1, $x2,$y2) = @_;
    my $n = $path->xyxy_to_n($x1,$y1, $x2,$y2);
    return (defined $n && $n < $n_hi ? $n : undef);
  };

 N: foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($x3,$y3) = ($x2+$rx, $y2+$ry);
    my ($x4,$y4) = ($x +$rx, $y +$ry);

    my $n8 = $xyxy_to_n->($x2,$y2, $x3,$y3);
    my $n9 = $xyxy_to_n->($x3,$y3, $x4,$y4);
    my $n10 = $xyxy_to_n->($x4,$y4, $x, $y);

    if (! defined $n8 && defined $n9 && ! defined $n10) {
      print "found N=$n, $n9 at $x,$y  dxdy=$dx,$dy\n";
    }
  }
  exit 0;
}

{
  # GpcClip

  my $path = Math::PlanePath::CCurve->new;
  foreach my $visits (4,1,2,3) {
    my $want = factorial($visits);
    print "$visits visits want $want orders\n";
    my %seen;
    for (my $n = 0; ; $n++) {
      my ($x,$y) = $path->n_to_xy($n);
      my @n_list;
      foreach my $dir4 (0 .. 3) {
        push @n_list,
          $path->xyxy_to_n_list($x,$y,
                                $x+$dir4_to_dx[$dir4], $y+$dir4_to_dy[$dir4]);
      }
      ### @n_list
      @n_list == $visits or next;
      my $order = n_list_to_order_string(@n_list);
      next if $seen{$order}++;
      print "$order at ",join(',',@n_list),"\n";
      last if scalar(keys %seen) >= $want;
    }
  }
  exit 0;

  sub n_list_to_order_string {
    return join(',', n_list_to_order(@_));
  }
  sub n_list_to_order {
    my %pos = map {$_[$_] => $_} 0 .. $#_;
    return map {$pos{$_}} sort @_;
  }
  CHECK {
    n_list_to_order_string(5,6,7) eq '0,1,2' or die;
    n_list_to_order_string(6,5,7) eq '1,0,2' or die;
  }
  sub factorial {
    my ($n) = @_;
    my $ret = 1;
    foreach my $i (2 .. $n) { $ret *= $i; }
    return $ret;
  }
}

{
  # GpcClip

  require Math::Geometry::Planar;
  my $planar1 = Math::Geometry::Planar->new;
  $planar1->points([ [0,0], [2,0], [2,1] ]);

  my $planar2 = Math::Geometry::Planar->new;
  $planar2->points([ [0,0], [0,1], [2,1] ]);

  my $gpc1 = $planar1->convert2gpc;
  my $gpc2 = $planar2->convert2gpc;

  my $gpc = Math::Geometry::Planar::GpcClip ("UNION", $gpc1, $gpc2);
  my @planars = Math::Geometry::Planar::Gpc2Polygons($gpc);
  ### @planars

  exit 0;
}

{
  # V1s3 recurrence non-negative poly18
  # (no good)

  my @r = (-1, 5, -12, 21, -28, 25, -8, -19, 53, -90,
           112, -118, 108, -88, 48, -16, -8, 32, -32);

  my @pending = ([ [0, @r[1..$#r]], [1] ]);
  my %seen;
  ### @pending
  for (;;) {
    my $elem = shift @pending // last;
    my ($terms,$factors) = @$elem;
    my $terms_str = join(',',@$terms);
    next if $seen{$terms_str}++;
    print "terms[",scalar(@$terms),"]: $terms_str\n";
    
    my @pos_list = rec_goes_negative_pos($terms);
    ### @pos_list
    if (! @pos_list) {
      print "all positive\n";
      print join(',',@$terms),"\n";
      print join(',',@$factors),"\n";
      last;
    }
    foreach my $pos (@pos_list) {
      my @new_terms = @$terms;
      my @new_factors = @$factors;
      add_at_pos(\@new_terms, $pos, \@r);
      $new_factors[$pos]++;
      next if @new_terms > 25;
      push @pending, [ \@new_terms, \@new_factors ];
    }
    print "\n";
  }
  exit 0;

  sub add_at_pos {
    my ($aref, $pos, $bref) = @_;
    foreach my $i (0 .. $#$bref) {
      $aref->[$pos+$i] += $bref->[$i];
    }
  }
  sub rec_goes_negative_pos {
    my ($aref) = @_;
    my $total;
    my @pos_list = ();
    foreach my $i (0 .. $#$aref) {
      my $r = $aref->[$i];
      if ($r > 0) { push @pos_list, $i; }
      $total += $r;
      if ($total < 0) { return @pos_list; }
    }
    return;
  }
}

{
  # squares 4-side outward
  #          1, 2, 5, 10, 23, 50, 109, 230, 499, 1070, 2281, 4826, 10199
  # cf QI4 = 1, 4, 19, 54, 154, 394, 1022, 2492
  #     *4 = 4, 16, 76, 216, 616, 1576, 4088, 9968

  # squares 2-side outward
  # 0,1,2,4,8,14,28,54,108,210,412,806,1588,3098,6052,11822,23116,45138,


  # vector(16,k,k--; 2*QI3(k) + 4*QI4(k))
  # 0, 0, 0, 0, 0, 0, 0, 0, 4, 12, 40, 116, 324, 792, 1912, 4424

  # vector(16,k,k--; QI2_adjacent(k) + 2*QI3(k) + 4*QI4(k))
  # 0, 0, 0, 1, 2, 5, 10, 23, 50, 109, 230, 499, 1070, 2281, 4826, 10199

  # conf_is_QI2_opposing(conf) = conf[1] && conf[9] && !conf[8] && !conf[10];
  # conf_is_QI2_adjacent(conf) = conf[1] && conf[8] && !conf[9] && !conf[10];
  # gQI2_adjacent(x) = conf_predicate_to_gf(conf_is_QI2_adjacent);
  # QI2_adjacent(k) = gf_term(gQI2_adjacent(x), k);
  # vector(100,k,k--; QI2_adjacent(k))
  # gf_to_linear_recurrence_poly(gQI2_adjacent(x)) == poly9(x) * poly8alt(x)

  # gQO4(x) = gQI2_adjacent(x) + 2*gQI3(x) + 4*gQI4(x)
  # gf_to_linear_recurrence_poly(gQO4(x)) == (x-2) * poly9(x)

  #                      even
  #                       ^
  #                       |
  #                       |
  #         even ------> odd   turn(odd) = +1 always
  #
  # any outward square is 2 or 4 sides

  # conf_is_QO4(conf) = conf[1] && conf[4] && conf[14];
  # conf_is_QO2(conf) = conf[1] && conf[4] && !conf[14];
  # gQO2(x) = conf_predicate_to_gf(conf_is_QO2);
  # gQO4(x) = conf_predicate_to_gf(conf_is_QO4) / 4;
  # QO2(k) = gf_term(gQO2(x), k);
  # QO4(k) = gf_term(gQO4(x), k);
  # vector(18,k,k--; QO2(k))
  # vector(18,k,k--; QO4(k))
  # gf_to_linear_recurrence_poly(gQO2(x)) == poly9(x)
  # gf_to_linear_recurrence_poly(gQO4(x)) == poly9(x) * (x-2)
  # vector(18,k,k--; if(k==0,0, S(k-1)))
  # vector(18,k,k--; D(k))


  my $path = Math::PlanePath::CCurve->new;
  foreach my $k (0 .. 17) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    my @count = (0,0,0,0,0);
  N: foreach my $n ($n_lo .. $n_hi-1) {
      #
      # x4,y4 <---------  x3,y3
      #   |               ^
      #   |               |
      #   |               |
      #   v               |
      #  x,y --------->  x2,y2
      #
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);

      my ($x2,$y2) = ($x+$dx, $y+$dy);
      my ($x3,$y3) = ($x2+$lx, $y2+$ly);
      my ($x4,$y4) = ($x +$lx, $y +$ly);

      my @n_list = ($path->xyxy_to_n($x2,$y2, $x3,$y3),
                    $path->xyxy_to_n($x3,$y3, $x4,$y4),
                    $path->xyxy_to_n($x4,$y4, $x, $y));
      my $num_sides = 1;
      foreach my $other_n (@n_list) {
        if (defined $other_n && $other_n < $n_hi) {
          if ($other_n > $n) {
            next N;  # count only with x,y segment the smallest
          }
          $num_sides++;
        }
      }

      # print "found N=$n, $n_list[0], $n_list[1], $n_list[2] at $x,$y  dxdy=$dx,$dy\n";
      $count[$num_sides]++;
    }
    shift @count;
    print "k=$k  count ",join(',',@count),"\n";
  }

  exit 0;
}

#------------------------------------------------------------------------

# expanded                1  2  3  4  5   6  7 8    9 19 11  12,13,14,15
use constant conf_left_perm  =>
  [undef,  8, 1, 9, 8,10,  9, 1,10, 15, 3, 2,  5, 4, 7, 6 ];
use constant conf_right_perm =>
  [undef, 10, 1,12,11,14, 13, 2,15,  3, 8, 1,  9, 8,10, 9 ];
use constant conf_perms_list => (conf_left_perm(), conf_right_perm());

#                         1  2   3  4  5   6  7  8  9 19 11  12,13,14,15
use constant conf_mirror_perm =>
  [undef, 1, 2, 15,14,13, 12,11,10, 9, 8, 7,  6, 5, 4, 3 ];
# my $identity_perm = [undef, 1 .. 15];

my $conf_to_index_locked = 0;
my %conf_to_index;
my @index_to_conf;
my $want_mirror_merge = 1;

sub conf_string {
  my ($conf, $n) = @_;
  return join('', map {conf_test($conf,$_) ? '1' : '0'} 1 .. 15);
}

sub conf_test {
  my ($conf, $n) = @_;
  return $conf & (1 << ($n-1));
}
sub conf_num_segments {
  my ($conf) = @_;
  return sum(map {conf_test($conf,1) ? 1 : 0} 1 .. 15);
}
# conf_make(a,b,c,...) segment positions to set
sub conf_make {
  my $conf = 0;
  foreach my $n (@_) {
    $conf |= (1 << ($n-1));
  }
  return $conf;
}

sub conf_square_num_sides {
  my ($conf) = @_;
  return sum(map {conf_test($conf,$_) ? 1 : 0} 1,8,9,10);
}
sub conf_is_double_seg {
  my ($conf) = @_;
  return conf_test($conf,1) && conf_test($conf,2);
}

sub conf_print {
  my ($conf) = @_;

  print "     .", (conf_test($conf,9) ? "--->" : "    "), ".\n";
  print "    ",
    (conf_test($conf,7) ? "|" : " "),
    (conf_test($conf,8) ? "^" : " "),
    "    ",
    (conf_test($conf,10) ? "|" : " "),
    (conf_test($conf,11) ? "^" : " "),
    "\n";
  print "    ",
    (conf_test($conf,7) ? "|" : " "),
    (conf_test($conf,8) ? "|" : " "),
    "    ",
    (conf_test($conf,10) ? "|" : " "),
    (conf_test($conf,11) ? "|" : " "),
    "\n";
  print "    ",
    (conf_test($conf,7) ? "v" : " "),
    (conf_test($conf,8) ? "|" : " "),
    "    ",
    (conf_test($conf,10) ? "v" : " "),
    (conf_test($conf,11) ? "|" : " "),
    "\n";

  print "", (conf_test($conf,6) ? "<---" : "    ")," .",
    (conf_test($conf,1) ? "<---" : "    "),". ",
    (conf_test($conf,12) ? "<---" : "    "),
    "  ",conf_string($conf),
    ", square=",conf_square_num_sides($conf),
    (conf_is_double_seg($conf),", double",""),
    "\n";
  print "", (conf_test($conf,5) ? "--->" : "    "),"  ",
    (conf_test($conf,2) ? "--->" : "    "),"  ",
    (conf_test($conf,13) ? "--->" : "    "),
    "\n";

  print "    ",
    (conf_test($conf,4) ? "|" : " "),
    (conf_test($conf,3) ? "^" : " "),
    "    ",
    (conf_test($conf,15) ? "|" : " "),
    (conf_test($conf,14) ? "^" : " "),
    "\n";
  print "    ",
    (conf_test($conf,4) ? "|" : " "),
    (conf_test($conf,3) ? "|" : " "),
    "    ",
    (conf_test($conf,15) ? "|" : " "),
    (conf_test($conf,14) ? "|" : " "),
    "\n";
  print "    ",
    (conf_test($conf,4) ? "v" : " "),
    (conf_test($conf,3) ? "|" : "."),
    "    ",
    (conf_test($conf,15) ? "v" : " "),
    (conf_test($conf,14) ? "|" : "."),
    "\n";
  print "-----------------------------------------------\n";
}

sub conf_permute {
  my ($conf, $perm) = @_;
  my $new_conf = 0;
  foreach my $n (1 .. 15) {
    if (conf_test($conf,$perm->[$n])) {
      $new_conf |= (1 << ($n - 1));
    }
  }
  return $new_conf;
}

sub conf_to_index {
  my ($conf) = @_;
  return $conf_to_index{$conf}
    // do {
      # new conf ...
      if ($conf_to_index_locked) {
        die "Unknown conf ",conf_string($conf);
      }

      push @index_to_conf, $conf;
      $conf_to_index{$conf} = $#index_to_conf;

      # when treating mirrors as equivalent insert also mirror image conf
      if ($want_mirror_merge) {
        $conf = conf_permute ($conf, conf_mirror_perm());
        $conf_to_index{$conf} = $#index_to_conf;
      }
      $#index_to_conf;
    };
}

use constant::defer conf_initial => conf_make(1);
use constant::defer conf_initials => sub {
  return map {conf_make($_)} 1 .. 15;
};
use constant::defer conf_initials_indexes => sub {
  return map {conf_to_index($_)} conf_initials();
};

# including the empty configuration
sub conf_create_all {
  return if $conf_to_index_locked;

  my @pending_confs = conf_initials();
  conf_initials_indexes();
    
  while (defined (my $conf = shift @pending_confs)) {
    # conf: conf_string($conf)."  count ".scalar(@index_to_conf)
    conf_to_index($conf);
    foreach my $perm (conf_perms_list()) {
      my $new_conf = conf_permute($conf, $perm);
      next if exists $conf_to_index{$new_conf};
      push @pending_confs, $new_conf;
    }
  }
  $conf_to_index_locked = 1;
}

my @index_expansion;
my @index_num_segments;
sub conf_create_index_expansion {
  my @perms = conf_perms_list();
  foreach my $index (0 .. $#index_to_conf) {
    my $conf = $index_to_conf[$index];
    $index_num_segments[$index] = conf_num_segments($conf);
    foreach my $bit (0,1) {
      my $new_conf = conf_permute($conf, $perms[$bit]);
      my $new_index = conf_to_index($new_conf);
      $index_expansion[$index]->[$bit] = $new_index;
    }
  }
}

sub conf_make_FSM {
  require MyFSM;
  my $table = {};
  foreach my $index (0 .. $#index_to_conf) {
    foreach my $bit (0,1) {
      my $new_index = $index_expansion[$index]->[$bit];
      $table->{$index}->{$bit} = $new_index;
    }
  }
  MyFSM->new (table   => $table,
              initial => conf_to_index(conf_initial()),
             );
}

sub conf_counts_iterate {
  my ($counts) = @_;
  my @new_counts;
  foreach my $index (0 .. $#$counts) {
    my $count = $counts->[$index] || next;
    foreach my $bit (0,1) {
      my $new_index = $index_expansion[$index]->[$bit];
      $new_counts[$new_index] += $count;
    }
  }
  @$counts = @new_counts;
  return $counts;
}

#------------------------------------------------------------------------

{
  # where is QI4 first 4-inward square

  my $path = Math::PlanePath::CCurve->new;
  my $k=11;
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
  print "k=$k  end $x_hi,$y_hi\n";

 N: foreach my $n ($n_lo .. $n_hi-1) {
    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    my ($rx,$ry) = xy_rotate_minus90($dx,$dy);

    my ($x2,$y2) = ($x+$dx, $y+$dy);
    my ($x3,$y3) = ($x2+$rx, $y2+$ry);
    my ($x4,$y4) = ($x +$rx, $y +$ry);

    my @n_list = ($path->xyxy_to_n($x2,$y2, $x3,$y3),
                  $path->xyxy_to_n($x3,$y3, $x4,$y4),
                  $path->xyxy_to_n($x4,$y4, $x, $y));
    foreach my $other_n (@n_list) {
      (defined $other_n && $other_n < $n_hi)
        or next N;
    }
    print "found N=$n, $n_list[0], $n_list[1], $n_list[2] at $x,$y  dxdy=$dx,$dy\n";
  }
  exit 0;
}

{
  # how many straight line crossings
  #
  #       |
  #       v
  # ----->----->
  #       |
  #       v
  #
  # with nothing else at the same point
  # 0 0 0 0 0 1 2 5 10 23 47 98 191 385 765 1518 2987 5912 11645 22956
  # with possible other segments
  # 0 0 0 0 0 1 2 5 10 24 50 108 222 473 984 2059 4261 8857 18285 37734

  my $path = Math::PlanePath::CCurve->new;
  my $n_straight_dir = sub {
    my ($n) = @_;
    my ($prev_dx,$prev_dy) = $path->n_to_dxdy($n-1);
    my ($dx,$dy) = $path->n_to_dxdy($n);
    unless ($dx == $prev_dx && $dy == $prev_dy) { return -1; }
    return dxdy_to_dir4($dx,$dy);
  };
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $n_half = $n_hi / 2;
    my $count = 0;
    my $count_straight = 0;
    foreach my $n ($n_lo+1 .. $n_hi-1) {
      my $dir = $n_straight_dir->($n);
      next unless $dir >= 0;

      my ($x,$y) = $path->n_to_xy($n);
      my @n_list = $path->xy_to_n_list($x,$y);
      # next unless @n_list == 2;        # for nothing else at the crossing
      next unless $n_list[0] == $n;

      foreach my $other_n (@n_list) {
        next if $other_n == $n;
        next unless $other_n < $n_hi;

        next unless $n < $n_half;         # only across two halves
        next unless $other_n >= $n_half;  # only across two halves

        my $other_dir = $n_straight_dir->($other_n);
        next unless $other_dir >= 0;
        if (dir4_are_right_angles($dir, $other_dir)) {
          if ($k <= 6) {
            print "at $x,$y\n";
          }
          $count++;
        } else {
          $count_straight++;
        }
      }
    }
    $count_straight /= 2;
    # print "k=$k  $count  straight $count_straight\n";
    print "$count\n";
    push @values, $count;
  }
  # require Math::OEIS::Grep;
  # Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # pairs of consecutive segments configurations
  #     ----->---->
  #     <----<-----
  #
  print join(',',expand_segment(0,0, 1,0)),"\n";
  join(',',expand_segment(0,0, 1,0)) eq '0,0,1,0,1,1' or die;
  join(',',expand_segment(0,0, 0,1)) eq '0,0,0,1,-1,1' or die;
  join(',',expand_segment(0,1, -1,1)) eq '-1,1,-2,1,-2,0' or die;

  sub consseg_print {
    my ($conf) = @_;
    # ### consseg_print(): $conf
    my $name = consseg_conf_to_name($conf) // '[no name]';
    print consseg_conf_string($conf),"  = $name\n";

    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
        $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$conf;
    my $chars = {};
    draw_segment($chars, $ax1,$ay1, $ax2,$ay2);
    draw_segment($chars, $ax2,$ay2, $ax3,$ay3);
    draw_segment($chars, $bx1,$by1, $bx2,$by2);
    draw_segment($chars, $bx2,$by2, $bx3,$by3);
    chars_print($chars);
  }

  sub consseg_conf_normalize {
    my ($conf) = @_;
    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
        $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$conf;

    $ax2 -= $ax1; $ay2 -= $ay1;  # to x1,y1 origin
    $ax3 -= $ax1; $ay3 -= $ay1;
    $bx1 -= $ax1; $by1 -= $ay1;
    $bx2 -= $ax1; $by2 -= $ay1;
    $bx3 -= $ax1; $by3 -= $ay1;
    $ax1 = 0; $ay1 = 0;
    until ($ax2 == -1 && $ay2 == 0) {     # to x2,y2=-1,0
      ($ax2,$ay2) = xy_rotate_minus90($ax2,$ay2);
      ($ax3,$ay3) = xy_rotate_minus90($ax3,$ay3);
      ($bx1,$by1) = xy_rotate_minus90($bx1,$by1);
      ($bx2,$by2) = xy_rotate_minus90($bx2,$by2);
      ($bx3,$by3) = xy_rotate_minus90($bx3,$by3);
    }
    ($ax2 == -1 && $ay2 == 0) or die;

    return [$ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
            $bx1,$by1, $bx2,$by2, $bx3,$by3];
  }
  sub consseg_conf_in_range {
    my ($conf) = @_;
    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
        $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$conf;

    return xyxy_xyxy_have_overlap($ax1,$ay1, $ax2,$ay2,
                                  $bx1,$by1, $bx2,$by2)
      || xyxy_xyxy_have_overlap($ax1,$ay1, $ax2,$ay2,
                                $bx2,$by2, $bx3,$by3)
      || xyxy_xyxy_have_overlap($ax2,$ay2, $ax3,$ay3,
                                $bx1,$by1, $bx2,$by2)
      || xyxy_xyxy_have_overlap($ax2,$ay2, $ax3,$ay3,
                                $bx2,$by2, $bx3,$by3);
  }

  sub consseg_conf_string {
    my ($conf) = @_;
    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
        $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$conf;
    return "$ax1,$ay1, $ax2,$ay2, $ax3,$ay3 and $bx1,$by1, $bx2,$by2, $bx3,$by3";
  }

  sub consseg_conf_children_raw {
    my ($conf) = @_;
    my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$conf;

    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
        $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$conf;

    my @aex = expand_two_segments($ax1,$ay1, $ax2,$ay2, $ax3,$ay3);
    my @bex = expand_two_segments($bx1,$by1, $bx2,$by2, $bx3,$by3);
    # x1,y1, x2,y2, x3,y3, x4,y4, x5,y6
    #  0      2      4

    my @ret;
    for (my $ai = 0; $ai <= 4; $ai+=2) {
      for (my $bi = 0; $bi <= 4; $bi+=2) {
        push @ret, [ $aex[$ai],$aex[$ai+1],
                     $aex[$ai+2],$aex[$ai+3],
                     $aex[$ai+4],$aex[$ai+5],
                     $bex[$bi],$bex[$bi+1],
                     $bex[$bi+2],$bex[$bi+3],
                     $bex[$bi+4],$bex[$bi+5] ];
      }
    }
    return @ret;
  }

  my $initial = [ 0,0, 1,0, 1,1,
                  1,1, 1,2, 0,2 ];
  my $initial_name = consseg_conf_to_name($initial);
  consseg_print($initial);

  my %consseg_conf_names;
  require Tie::IxHash;
  tie %consseg_conf_names, 'Tie::IxHash';
  $consseg_conf_names{consseg_conf_string($initial)} = 'initial';
  $consseg_conf_names{'0,0, -1,0, -2,0 and -1,-1, -1,0, -1,1'} = 'cross up';
  $consseg_conf_names{'0,0, -1,0, -2,0 and -1,1, -1,0, -1,-1'} = 'cross down';
  $consseg_conf_names{'0,0, -1,0, -2,0 and -2,0, -1,0, 0,0'} = 'pair';
  ### consseg_conf_names: keys %consseg_conf_names

  my $consseg_conf_to_name_upto = 0;
  sub consseg_conf_to_name {
    my ($conf) = @_;
    return $consseg_conf_names{consseg_conf_string($conf)}
      //= ++$consseg_conf_to_name_upto;
  }

  sub consseg_conf_children {
    my ($conf) = @_;
    return grep {consseg_conf_in_range($_)}
      map {consseg_conf_normalize($_)}
      consseg_conf_children_raw($conf);
  }

  xyxy_xyxy_have_overlap(0,0, -1,0, 1,0, 0,0) or die;

  my %matrix;
  require Graph::Easy;
  my $easy = Graph::Easy->new;

  my %seen;
  my @pending = ($initial);
  my $unnamed = 0;
  while (@pending) {
    my $parent = shift @pending;
    print "\n*******************\n";
    print "parent ",consseg_conf_to_name($parent) // '[no name]', "\n";
    {
      my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3,
          $bx1,$by1, $bx2,$by2, $bx3,$by3) = @$parent;
      my @aex = expand_two_segments($ax1,$ay1, $ax2,$ay2, $ax3,$ay3);
      my @bex = expand_two_segments($bx1,$by1, $bx2,$by2, $bx3,$by3);
      my $chars = {};
      foreach my $aref (\@aex, \@bex) {
        for (my $i = 0; $i+3 <= $#$aref; $i+=2) {
          draw_segment($chars,
                       $aref->[$i],$aref->[$i+1],
                       $aref->[$i+2],$aref->[$i+3]);
        }
      }
      print "expanded:\n";
      chars_print($chars);
    }

    my $parent_name = consseg_conf_to_name($parent);
    my @children = consseg_conf_children_raw($parent);
    foreach my $i (0 .. $#children) {
      my $conf = $children[$i];
      $conf = consseg_conf_normalize($conf);

      my $in_range = consseg_conf_in_range($conf);
      if ($in_range) {
        print "\nchild:  in range ",$in_range // '[not]', "\n";
        consseg_print($conf);
        my $str = consseg_conf_string($conf);
        my $name = consseg_conf_to_name($conf);

        $matrix{$parent_name}->{$name}++;
        ### matrix: "$parent_name to $name"

        $easy->add_edge_once($parent_name, $name);
        my $edge = $easy->edge($parent_name, $name);
        my $label = $edge->label || 0;
        $edge->label($label+1);

        if (! $seen{$str}++ && ! (defined $name && $seen{$name}++)) {
          push @pending, $conf;
          if (! defined $name) { $unnamed++; }
        }
      }
    }
  }
  print "total ",scalar(keys %seen)," confs, unnamed $unnamed\n";
  print "graph nodes ", scalar($easy->nodes),
    " edges ", scalar($easy->edges), "\n";

  {
    my @names = values %consseg_conf_names;
    open my $fh, '>', '/tmp/consseg-generated.gp' or die;

    print $fh "v = {[\n";
    foreach my $i (0 .. $#names) {
      my $value = ($i == 0 ? 1 : 0);
      print $fh $value;
      if ($i != $#names) { print $fh ","; }
    }
    print $fh "]~};\n";

    print $fh "m = {[\n";
    foreach my $i (0 .. $#names) {
      my $from = $names[$i];
      my $row = $matrix{$from};
      ### $from
      ### $row
      foreach my $j (0 .. $#names) {
        print $fh $row->{$names[$j]} || 0;
        if ($j != $#names) { print $fh ","; }
      }
      if ($i != $#names) { print $fh ";"; }
      print $fh "\n";
    }
    print $fh "]};\n";
    close $fh or die;
  }

  my %want = (
              # 'cross' => 1,
              'cross up' => 1,
              'cross down' => 1,
              $initial_name => 1,
             );
  my @nodes = $easy->nodes;
  my $more;
  do {
    $more = 0;
    foreach my $from (keys %matrix) {
      next if $want{$from};
      my $row = $matrix{$from};
      foreach my $to (keys %$row) {
        if ($want{$to}) {
          $want{$from} = 1;
          $more = 1;
        }
      }
    }
  } while ($more);
  print "reachable want ", scalar(keys %want), "\n";

  # MyGraphs::Graph_Easy_view($easy);

  # require MyFSM;
  # my $fsm = MyFSM->new
  #   (table => \%table,
  #    # accepting => { 8 => 'non' },
  #    initial   => consseg_conf_to_name($initial),
  #   );
  # $fsm->view;


  exit 0;

  sub expand_two_segments {
    my ($x,$y, $x2,$y2, $x3,$y3) = @_;
    my @ret = expand_segment($x,$y, $x2,$y2);
    pop @ret; pop @ret;
    return (@ret, expand_segment($x2,$y2, $x3,$y3));
  }
  sub expand_segment {
    my ($x,$y, $x2,$y2) = @_;
    my $dx = $x2 - $x;
    my $dy = $y2 - $y;
    my ($bx,$by) = mul($x,$y);
    my ($bx2,$by2) = mul($x2,$y2);
    return ($bx,$by, $bx+$dx,$by+$dy, $bx2,$by2);
  }
}

{
  # singles and doubles by all configurations
  conf_create_all();
  conf_create_index_expansion();

  my @counts;
  foreach my $index (conf_initials_indexes()) {
    $counts[$index] += 1;
  }
  my @is_double
    = map {conf_is_double_seg($index_to_conf[$_])} 0 .. $#index_to_conf;
  foreach my $k (0 .. 20) {
    my $S = 0;
    my $D = 0;
    foreach my $index (0 .. $#counts) {
      my $count = $counts[$index] || next;
      if ($is_double[$index]) {
        $D += $count;
        if ($k < 8) {
          my $conf = $index_to_conf[$index];
          my $conf_str = conf_string($conf);
          print "  doubles $count at $index $conf_str\n";
        }
      } else {
        $S += $count * $index_num_segments[$index];
      }
    }
    $S /= 15;
    $D /= 2;
    print "k=$k  $S $D\n";
    conf_counts_iterate(\@counts);
  }
  ### @counts
  exit 0;
}

{
  # double-visited segment configurations graph

  require Graph::Easy;
  my $easy = Graph::Easy->new;

  $easy->add_edge(1, 3, 'x2');
  $easy->add_edge(1, 7);
  $easy->add_edge(2, 3, 'x2');
  $easy->add_edge(3, 10);
  $easy->add_edge(4, 7, 'x2');
  $easy->add_edge(5, 4);
  $easy->add_edge(5, 10, 'x2');
  $easy->add_edge(6, 11, 'x2');
  $easy->add_edge(7, 1);
  $easy->add_edge(7, 6);
  $easy->add_edge(7, 9, 'x2');
  $easy->add_edge(8, 5);
  $easy->add_edge(8, 8);
  $easy->add_edge(9, 2);
  $easy->add_edge(9, 9);
  $easy->add_edge(10, 8);
  $easy->add_edge(11, 1);
  # $easy->add_edge(12, 12);

  $easy->set_attribute('flow','east');
  $easy->set_attribute('root','11'); # for as_graphviz()
  $easy->{att}->{root} = '11';       # for root_node() for as_ascii()

  MyGraphs::Graph_Easy_view($easy);
  exit 0;
}

{
  # double-traversed segments configurations

  sub draw_segment {
    my ($chars, $x1,$y1, $x2,$y2) = @_;
    my $dx_1 = $x2 - $x1;
    my $dy_1 = $y2 - $y1;
    $x1 *= 6;
    $x2 *= 6;
    $y1 *= 4;
    $y2 *= 4;
    $x1 += 2*$dx_1;
    $y1 += 2*$dy_1;
    $x2 -= $dx_1;
    $y2 -= $dy_1;
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    if ($dx < 0) { $y1++; $y2++; }
    if ($dy > 0) { $x1++; $x2++; }
    ($dx == 0 || $dy == 0) or die "cannot draw dxdy = $dx,$dy";
    if ($dx) {
      foreach my $x (min($x1,$x2) .. max($x1,$x2)) {
        $chars->{$y1}->{$x} = '-';
      }
      if ($dx > 0) {
        $chars->{$y1}->{$x2} = '>';
      } else {
        $chars->{$y1}->{$x2} = '<';
      }
    }
    if ($dy) {
      foreach my $y (min($y1,$y2) .. max($y1,$y2)) {
        $chars->{$y}->{$x1} = '|';
      }
      if ($dy > 0) {
        $chars->{$y2}->{$x1} = '^';
      } else {
        $chars->{$y2}->{$x1} = 'v';
      }
    }
  }

  sub chars_print {
    my ($chars) = @_;
    my $x_min = min(0, map {keys %$_} values %$chars);

    foreach my $y (reverse min(keys %$chars) .. max(keys %$chars)) {
      my $aref = $chars->{$y} // {};
      foreach my $x ($x_min .. max(0,keys %$aref)) {
        print $aref->{$x} // ' ';
      }
      print "\n";
    }
  }

  sub doubleseg_print {
    my ($conf) = @_;
    # ### doubleseg_print(): $conf
    my $name = doubleseg_conf_to_name($conf) // '[no name]';
    print doubleseg_conf_string($conf),"  = $name\n";

    my ($x1,$y1, $x2,$y2, $x3,$y3, $x4,$y4) = @$conf;
    my $chars = {};
    draw_segment($chars, $x1,$y1, $x2,$y2);
    draw_segment($chars, $x3,$y3, $x4,$y4);
    chars_print($chars);
  }

  sub doubleseg_conf_normalize {
    my ($conf) = @_;
    my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$conf;
    ### norm: "$x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4"
    $x2 -= $x1; $y2 -= $y1;  # to x1,y1 origin
    $x3 -= $x1; $y3 -= $y1;
    $x4 -= $x1; $y4 -= $y1;
    $x1 = 0; $y1 = 0;
    until ($x2 == -1 && $y2 == 0) {     # to x2,y2=-1,0
      ($x2,$y2) = xy_rotate_minus90($x2,$y2);
      ($x3,$y3) = xy_rotate_minus90($x3,$y3);
      ($x4,$y4) = xy_rotate_minus90($x4,$y4);
    }
    ($x2 == -1 && $y2 == 0) or die;

    if ($x2-$x1 == $x4-$x3 && $y2-$y1 == $y4-$y3
        && $y3 < min($y1,$y2) && $y4 < min($y1,$y2)) {
      ($x1,$y1, $x3,$y3) = ($x3,$y3, $x1,$y1);
      ($x2,$y2, $x4,$y4) = ($x4,$y4, $x2,$y2);

      $x2 -= $x1; $y2 -= $y1;  # to x1,y1 origin
      $x3 -= $x1; $y3 -= $y1;
      $x4 -= $x1; $y4 -= $y1;
      $x1 = 0; $y1 = 0;
    }

    if ($x3 < 0 && $x4 < 0) {
      # mirror image
      ($x3,$x4) = (-$x4 - 1, -$x3 - 1);
      ($y3,$y4) = ($y4,$y3);              # vertical opposite expansion side
    }
    return [$x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4];
  }
  sub doubleseg_conf_in_range {
    my ($conf) = @_;
    $conf = doubleseg_conf_normalize($conf);
    my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$conf;
    ### in range: "$x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4"
    return xyxy_xyxy_have_overlap($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4);
  }

  sub doubleseg_conf_string {
    my ($conf) = @_;
    my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$conf;
    return "$x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4";
  }

  sub doubleseg_conf_children_raw {
    my ($conf) = @_;
    my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$conf;

    my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3) = expand_segment($x1,$y1, $x2,$y2);
    my ($bx1,$by1, $bx2,$by2, $bx3,$by3) = expand_segment($x3,$y3, $x4,$y4);

    return ([ $ax1,$ay1, $ax2,$ay2,  $bx1,$by1, $bx2,$by2 ],
            [ $ax1,$ay1, $ax2,$ay2,  $bx2,$by2, $bx3,$by3 ],
            [ $ax2,$ay2, $ax3,$ay3,  $bx1,$by1, $bx2,$by2 ],
            [ $ax2,$ay2, $ax3,$ay3,  $bx2,$by2, $bx3,$by3 ],
           );
  }

  my %doubleseg_conf_names = ('0,0, -1,0,  1,0, 0,0' => 'p1 line',
                              '0,0, -1,0,  0,1, -1,1' => 'p2 above',
                              '0,0, -1,0,  1,1, 0,1' => 'p3 offset',
                              '0,0, -1,0,  -1,-1, 0,-1' => 'p4 above opposites',
                              '0,0, -1,0,  -1,1, 0,1' => 'p4 above opposites',
                              '0,0, -1,0,  0,1, 1,1'    => 'p5 above away',
                              '0,0, -1,0,  -1,0, 0,0' => 'p6 double',
                              '0,0, -1,0,  -1,0, -1,1' => 'p7 bend',
                              '0,0, -1,0,  0,1, 0,0'   => 'p7 bend',
                              '0,0, -1,0,  0,-1, 0,0'  => 'p7 bend', # mirror
                              '0,0, -1,0,  1,1, 1,0'   => 'p8 away other',
                              '0,0, -1,0,  0,2, 0,1'   => 'p8 away other',
                              '0,0, -1,0,  0,0, 0,1' => 'p9 outward 90',
                              '0,0, -1,0,  0,0, 0,-1' => 'p9 outward 90',
                              '0,0, -1,0,  1,0, 1,-1' => 'p10 apart',
                              '0,0, -1,0,  2,-1, 2,0' => 'p10 apart',
                              '0,0, -1,0,  0,1, 0,2'  => 'p10 apart',
                              '0,0, 1,0,  1,0, 1,1' => 'p11 initial',
                              '0,0, -1,0,  0,0, 1,0' => 'p12 away',
                             );
  sub doubleseg_conf_to_name {
    my ($conf) = @_;
    return $doubleseg_conf_names{doubleseg_conf_string($conf)};
  }

  sub doubleseg_conf_children {
    my ($conf) = @_;
    return grep {doubleseg_conf_in_range($_)}
      map {doubleseg_conf_normalize($_)}
      doubleseg_conf_children_raw($conf);
  }

  my $initial = [0,0, 1,0,  1,0, 1,1];
  doubleseg_print($initial);

  xyxy_xyxy_have_overlap(0,0, -1,0, 1,0, 0,0) or die;

  my @pending = ($initial);
  my %seen;
  my $unnamed = 0;
  while (@pending) {
    my $parent = shift @pending;
    print "\n*******************\n";
    print "parent ",doubleseg_conf_to_name($parent) // '[no name]', "\n";
    {
      my ($x1,$y1, $x2,$y2,  $x3,$y3, $x4,$y4) = @$parent;
      my ($ax1,$ay1, $ax2,$ay2, $ax3,$ay3) = expand_segment($x1,$y1, $x2,$y2);
      my ($bx1,$by1, $bx2,$by2, $bx3,$by3) = expand_segment($x3,$y3, $x4,$y4);
      my $chars = {};
      draw_segment($chars, $ax1,$ay1, $ax2,$ay2);
      draw_segment($chars, $ax2,$ay2, $ax3,$ay3);
      draw_segment($chars, $bx1,$by1, $bx2,$by2);
      draw_segment($chars, $bx2,$by2, $bx3,$by3);
      print "expanded:\n";
      chars_print($chars);
    }

    foreach my $conf (doubleseg_conf_children_raw($parent)) {
      $conf = doubleseg_conf_normalize($conf);
      my $in_range = doubleseg_conf_in_range($conf);
      print "\nchild:  in range ",$in_range // '[not]', "\n";
      doubleseg_print($conf);
      my $str = doubleseg_conf_string($conf);
      my $name = doubleseg_conf_to_name($conf);
      if ($in_range && ! $seen{$str}++ && ! (defined $name && $seen{$name}++)) {
        push @pending, $conf;
        if (! defined $name) { $unnamed++; }
      }
    }
  }
  print "total ",scalar(keys %seen)," confs, unnamed $unnamed\n";
  exit 0;
}

{
  # convex hull overlaps tikz individual configurations
  my $scale = .5;
  open my $tex_fh, '>', '/tmp/overlap.tex' or die;

  print $tex_fh <<"HERE";
\\documentclass[a4paper]{article}
\\pagestyle{empty}
\\usepackage{tikz}
\\begin{document}
\\begin{tikzpicture}[scale=$scale]
HERE
  my $x = 0;
  my $y = 0;

  my $href = MyCCurve::xyxy_convex_hull_overlap();
  foreach my $key (keys %$href) {
    my ($x1,$y1, $x2,$y2) = split /,\s*/, $key;
    my $dx = $x2 - $x1;
    my $dy = $y2 - $y1;
    if ($dx > 0) { $y1 += .1; $y2 += .1; }
    if ($dy > 0) { $x1 += .1; $x2 += .1; }
    print $tex_fh "  \\draw[thick,->] ($x+0,$y+0)--($x+-1,$y+0)\n";
    print $tex_fh "    node[midway,below] {0};\n";
    print $tex_fh "  \\draw[thick,->] ($x+$x1,$y+$y1)--($x+$x2,$y+$y2);\n";

    $x += 4;
    if ($x > 20) {
      $x = 0;
  print $tex_fh <<"HERE";
\\end{tikzpicture}

\\vspace{3\\baselineskip}
\\begin{tikzpicture}[scale=$scale]
HERE
    }
  }
  print $tex_fh <<'HERE';
\end{tikzpicture}
\end{document}
HERE
  close $tex_fh or die;

  require IPC::Run;
  IPC::Run::run(['cat','/tmp/overlap.tex']) or die;
  chdir '/tmp' or die;
  IPC::Run::run(['latexmk','-pdf','overlap.tex'], '<',File::Spec->devnull)
      or die;;
  IPC::Run::run(['xpdf', '/tmp/overlap.pdf'],
                '<', File::Spec->devnull)
      or die;
  exit 0;
}




{
  # FSM of all configurations

  conf_create_all();
  conf_create_index_expansion();
  my $fsm = conf_make_FSM();
  print "total configurations ", scalar(@index_to_conf), "\n";

  foreach my $index (0 .. $#index_to_conf) {
    my $conf = $index_to_conf[$index];
    # if (conf_is_double_seg($conf)) {
    #   $fsm->{'accepting'}->{$index} = 'double';
    # }
    if (conf_square_num_sides($conf) == 2) {
      $fsm->{'accepting'}->{$index} = 'square';
    }
  }
  print "accepting ",scalar(keys %{$fsm->{'accepting'}}),"\n";

  print "simplify\n";
  $fsm->simplify (verbose => 1);
  print "view\n";
  $fsm->view;
  exit 0;
}


{
  # all configurations
  conf_create_all();
  print "total configurations ", scalar(@index_to_conf), "\n";
  exit 0;
}
{
  # permute
  my $conf = conf_initial();
  conf_print ($conf);
  foreach my $perm (conf_perms_list()) {
    my $new_conf = conf_permute($conf, $perm);
    print "permute:\n";
    conf_print($new_conf);
  }
  exit 0;
}
{
  # initial configurations
  foreach my $conf (conf_initials()) {
    conf_print ($conf);
  }
  exit 0;
}




{
  # drawing with Language::Logo

  require Language::Logo;
  require Math::NumSeq::Fibbinary;
  my $fibbinary = Math::NumSeq::Fibbinary->new;

  my $lo = Logo->new(update => 20, port=>8222);
  $lo->command("pendown");
  foreach my $n (1 .. 2560) {
    my $b = $n;

    # variation fibbinary bit count is wider type C curve
    $b = $fibbinary->ith($b);

    # my $turn4 = CountLowZeros($b) - 1;
    # my $turn360 = $turn4 * 90;
    # $lo->command("forward 3; right $turn360");

    my $dir4 = count_1_bits($b) - 1;
    my $dir360 = $dir4 * 90;
    $lo->command("forward 3; seth $dir360");
  }
  $lo->disconnect("Finished...");
  exit 0;

  sub count_1_bits {
    my ($n) = @_;
    my $count = 0;
    while ($n) {
      $count += ($n & 1);
      $n >>= 1;
    }
    return $count;
  }
}


{
  # GR centroid of right boundary segments
  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my $xy_rotate = sub {
      my ($x,$y) = @_;
      for (1 .. $k+4) {
        ($x,$y) = xy_rotate_minus45($x,$y);
      }
      return ($x,$y);
    };

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $len = hypot($x_hi,$y_hi);

    my $x_total = 0;
    my $y_total = 0;
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      next unless is_right_boundary_HtoL($n,$k);
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      ($x,$y) = $xy_rotate->($x,$y);
      ($dx,$dy) = $xy_rotate->($dx,$dy);
      $x_total += 2*$x + $dx;
      $y_total += 2*$y + $dy;
      $count++;
    }
    # ($x_total,$y_total) = $xy_rotate->($x_total,$y_total);

    my $gx = $x_total / $count / $len / 2;
    my $gy = $y_total / $count / $len / 2;
    print "k=$k  $gx $gy\n";
    # print "k=$k  $gx ",1/(.5-$gy),"\n";
    if ($k % 2 == 0) {
      push @values, $y_total;
    }
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # area of part by polygon
  require Math::Geometry::Planar;
  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 26) {
    my @points;

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    $n_lo = PartN($k);
    $n_hi = PartN($k+1);
    my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);

    my $is_traversed = sub {
      my ($x,$y, $x2,$y2) = @_;
      foreach my $n ($path->xyxy_to_n_list_either($x,$y, $x2,$y2)) {
        if ($n <= $n_hi) { return 1; }
      }
      return 0;
    };

    my ($target_x, $target_y) = ($x_hi,$y_hi);
    my ($x,$y) = ($x_lo,$y_lo);
    my ($dx,$dy) = ($n_lo > 0 ? $path->n_to_dxdy($n_lo-1) : (1,0));
    foreach (1, 2) {
      until ($x == $target_x && $y == $target_y) {
        ### at: "$x,$y dir $dx,$dy"
        push @points, [$x,$y];

        my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
        my $x2 = $x + $lx;
        my $y2 = $y + $ly;
        ### consider left: "$x2, $y2"
        if ($is_traversed->($x,$y, $x2,$y2)) {
          ($x,$y) = ($x2,$y2);
          ($dx,$dy) = ($lx,$ly);
          next;
        }

        $x2 = $x + $dx;
        $y2 = $y + $dy;
        ### consider straight: "$x2, $y2"
        if ($is_traversed->($x,$y, $x2,$y2)) {
          ($x,$y) = ($x2,$y2);
          next;
        }

        my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
        $x2 = $x + $rx;
        $y2 = $y + $ry;
        ### consider right: "$x2, $y2"
        if ($is_traversed->($x,$y, $x2,$y2)) {
          ($x,$y) = ($x2,$y2);
          ($dx,$dy) = ($rx,$ry);
          next;
        }

        $x2 = $x - $dx;
        $y2 = $y - $dy;
        $is_traversed->($x,$y, $x2,$y2) or die;
        ($x,$y) = ($x2,$y2);
        ($dx,$dy) = (-$dx,-$dy);
      }
      ($target_x, $target_y) = ($x_lo,$y_lo);
    }

    $dx = $x_lo - $x;
    $dy = $y_lo - $y;
    if ($dx != 0 && $dy != 0) {
      abs($dx) == abs($dy) or die;
    }
    if ($dx) { $dx /= abs($dx); }
    if ($dy) { $dy /= abs($dy); }
    until ($x == $x_lo && $y == $y_lo) {
      push @points, [$x,$y];
      $x += $dx;
      $y += $dy;
    }

    my $area = 0;
    if (@points >= 3) {
      my $planar = Math::Geometry::Planar->new;
      $planar->points(\@points);
      $area = $planar->area;
    }

    print "k=$k  $area\n";
    push @values, $area;
  }

  require Math::OEIS::Grep;
  print join(',',@values),"\n";
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # convex hull

  my $points = [ [0,0], [1,0], ];
  my $ex = 1;
  my $ey = 0;
  my @values;
  foreach my $k (0 .. 40) {
    print "$k  ",points_str($points),"\n";
    my $area = MyPlanar::points_to_area($points);
    push @values, $area;

    ($points,$ex,$ey) = hull_expand($points,$ex,$ey);
  }
  print join(',',@values),"\n";
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values);
  exit 0;
}

{
  # left boundary length by path, Lpart or whole L
  $|=1;
  my @L;
  my @turn_left;
  my @turn_straight;
  my @turn_right;
  my @turn_reverse;
  my @Z;
  foreach my $k (0 .. 20) {
    # {
    #   my $path = Math::PlanePath::CCurve->new;
    #   my $n_lo = PartN($k);
    #   my $n_hi = PartN($k+1);
    #   my ($x_lo,$y_lo) = $path->n_to_xy($n_lo);
    #   my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    #   print "k=$k N=$n_lo to $n_hi  ($x_lo,$y_lo) to ($x_hi,$y_hi)\n";
    #   print "\n";
    # }

    my ($L, $turn_left,$turn_straight,$turn_right,$turn_reverse, $Z)
      = L_by_path($k);
    push @L, $L;
    push @turn_left, $turn_left;
    push @turn_straight,$turn_straight;
    push @turn_right,$turn_right;
    push @turn_reverse,$turn_reverse;
    push @Z, $Z;
    # my $scaled = ($L - 2*$Z + sqrt(2)*$Z) / sqrt(2)**$k;
    my $scaled = ($L + (sqrt(2) - 2)*$turn_left) / sqrt(2)**$k;
    printf "k=%2d  %5d  %5d %5d %5d %5d  %5d %.8f\n",
      $k, $L, $turn_left,$turn_straight,$turn_right,$turn_reverse, $Z, $scaled;


  }
  require Math::OEIS::Grep;
  print join(',',@L),"\n";
  Math::OEIS::Grep->search(array => \@L, verbose=>1);
  Math::OEIS::Grep->search(array => \@turn_left, verbose=>1);
  Math::OEIS::Grep->search(array => \@turn_straight, verbose=>1);
  Math::OEIS::Grep->search(array => \@turn_right, verbose=>1);
  Math::OEIS::Grep->search(array => \@turn_reverse, verbose=>1);
  Math::OEIS::Grep->search(array => \@Z, verbose=>1);
  exit 0;

  sub L_by_path {
    my ($k) = @_;
    ### L_by_path: $k
    my $path = Math::PlanePath::CCurve->new;

    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    $n_lo = PartN($k);
    $n_hi = PartN($k+1);

    my $is_traversed = sub {
      my ($x,$y, $x2,$y2) = @_;
      foreach my $n ($path->xyxy_to_n_list_either($x,$y, $x2,$y2)) {
        if ($n <= $n_hi) { return 1; }
      }
      return 0;
    };

    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $L = 0;
    my $Z = 0;
    my $turn_left = 0;
    my $turn_straight = 0;
    my $turn_right = 0;
    my $turn_reverse = 0;
    my ($x,$y) = $path->n_to_xy($n_lo);
    my ($dx,$dy) = ($n_lo > 0 ? $path->n_to_dxdy($n_lo-1) : (1,0));
    my $prev_is_right = 0;
    until ($x == $x_hi && $y == $y_hi) {
      ### at: "$x,$y dir $dx,$dy     L $L"
      $L++;

      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      my $x2 = $x + $lx;
      my $y2 = $y + $ly;
      ### consider left: "$x2, $y2"
      if ($is_traversed->($x,$y, $x2,$y2)) {
        $turn_left++;
        ($x,$y) = ($x2,$y2);
        ($dx,$dy) = ($lx,$ly);
        if ($prev_is_right) {
          $Z++;
        }
        $prev_is_right = 0;
        next;
      }

      $x2 = $x + $dx;
      $y2 = $y + $dy;
      ### consider straight: "$x2, $y2"
      if ($is_traversed->($x,$y, $x2,$y2)) {
        $turn_straight++;
        ($x,$y) = ($x2,$y2);
        $prev_is_right = 0;
        next;
      }

      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      $x2 = $x + $rx;
      $y2 = $y + $ry;
      ### consider right: "$x2, $y2"
      if ($is_traversed->($x,$y, $x2,$y2)) {
        $turn_right++;
        ($x,$y) = ($x2,$y2);
        ($dx,$dy) = ($rx,$ry);
        $prev_is_right = 1;
        next;
      }

      $x2 = $x - $dx;
      $y2 = $y - $dy;
      $is_traversed->($x,$y, $x2,$y2) or die;
      ($x,$y) = ($x2,$y2);
      ($dx,$dy) = (-$dx,-$dy);
      $turn_reverse++;
      $prev_is_right = 0;
    }
    return ($L, $turn_left,$turn_straight,$turn_right,$turn_reverse, $Z);
  }
}

{
  # right boundary length diagonals across zig-zag
  $|=1;  
  my @straight;
  my @diagonal;
  foreach my $k (0 .. 16) {
    my ($straight, $diagonal) = Rdiag_by_path($k);
    push @straight, $straight;
    push @diagonal, $diagonal;
    print "$straight+$diagonal*sqrt2, "
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@straight, verbose=>1);
  Math::OEIS::Grep->search(array => \@diagonal, verbose=>1);
  exit 0;

  sub Rdiag_by_path {
    my ($k) = @_;
    ### Rdiag_by_path: $k
    my $path = Math::PlanePath::CCurve->new;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my ($x_hi,$y_hi) = $path->n_to_xy($n_hi);
    my $straight = 0;
    my $diagonal = 0;

    my $is_visited = sub {
      my ($x,$y) = @_;
      foreach my $n ($path->xy_to_n_list($x,$y)) {
        if ($n <= $n_hi) { return 1; }
      }
      return 0;
    };

    my ($x,$y) = (0,0);
    my ($dx,$dy) = (1,0);
    until ($x == $x_hi && $y == $y_hi) {
      ### at: "$x,$y dir $dx,$dy     straight $straight diagonal $diagonal"
      my $x2 = $x + $dx;
      my $y2 = $y + $dy;
      ### consider straight: "$x2, $y2"
      if ($is_visited->($x2,$y2)) {
        $straight++;
        ($x,$y) = ($x2,$y2);
        next;
      }

      my ($lx,$ly) = xy_rotate_plus90($dx,$dy);
      $x2 += $lx;
      $y2 += $ly;
      ### consider diagonal: "$x2, $y2"
      if ($is_visited->($x2,$y2)) {
        ### diagonal ...
        ($x,$y) = ($x2,$y2);
        $diagonal++;
        next;
      }

      ### left turn 90 ...
      $x2 = $x + $lx;
      $y2 = $y + $ly;
      $is_visited->($x2,$y2) or die;
      ($x,$y) = ($x2,$y2);
      ($dx,$dy) = ($lx,$ly);
      $straight++;
    }
    return ($straight, $diagonal);
  }
}

{
  # triangles in regions drawing gray scale
  my %triangles;

  open my $fh, '>', '/tmp/cc.tex' or die;
  print $fh "\\documentclass{article}\n";
  print $fh "\\usepackage{tikz}\n";
  print $fh "\\usepackage[margin=.5cm]{geometry}\n";
  print $fh "\\begin{document}\n";

  my $weight_to_colour = sub {
    my ($t) = @_;
    $t /= 105;  # ranging 0 to 1 inclusive
    my $percent = int(20 + 60*$t);
    return "[black!$percent]";
  };

  $triangles{2}{-1} = 1;
  my ($ex,$ey) = (4,0);
  foreach my $level (0 .. 14) {
    print "k=$level\n";
    my $total = 0;

    print $fh "% total $total\n";
    print $fh "\\begin{center}\n\\begin{tikzpicture}[scale=1/sqrt(2)^$level,rotate=180-45*$level]\n";

    $total = 0;
    my %spray;
    foreach my $x (keys %triangles) {
      my $row = $triangles{$x};
      foreach my $y (keys %$row) {
        my $t = $row->{$y};
        $total += $t;

        my ($dx,$dy);
        if ($y % 4 == 1)    { ($dx,$dy) = (0,1) }
        elsif ($y % 4 == 3) { ($dx,$dy) = (0,-1) }
        elsif ($y % 4 == 2) {
          if ($x % 4 == 1)    { ($dx,$dy) = (1,0) }
          elsif ($x % 4 == 3) { ($dx,$dy) = (-1,0) }
          else { die; }
        } else { die; }

        my ($hx,$hy) = xy_rotate_plus90($dx,$dy);

        $spray{$x+2*$dx}{$y+2*$dy} += 24*$t;

        $spray{$x+$dx - 3*$hx}{$y+$dy - 3*$hy} += 16*$t;
        $spray{$x+$dx + 3*$hx}{$y+$dy + 3*$hy} += 16*$t;
        $spray{$x+$dx -   $hx}{$y+$dy -   $hy} += 8*$t;
        $spray{$x+$dx +   $hx}{$y+$dy +   $hy} += 8*$t;

        $spray{$x-4*$hx}{$y-4*$hy} += 8*$t;
        $spray{$x+4*$hx}{$y+4*$hy} += 8*$t;
        $spray{$x      }{$y      } += 2*$t;

        $spray{$x-2*$dx - 4*$hx}{$y-2*$dy - 4*$hy} += 4*$t;
        $spray{$x-2*$dx + 4*$hx}{$y-2*$dy + 4*$hy} += 4*$t;
        $spray{$x-2*$dx        }{$y-2*$dy        } += $t;

        $spray{$x-3*$dx - 3*$hx}{$y-3*$dy - 3*$hy} += 2*$t;
        $spray{$x-3*$dx + 3*$hx}{$y-3*$dy + 3*$hy} += 2*$t;
        $spray{$x-3*$dx -   $hx}{$y-3*$dy -   $hy} += $t;
        $spray{$x-3*$dx +   $hx}{$y-3*$dy +   $hy} += $t;
      }
    }
    print $fh "% total $total\n";

    $total = 0;
    my $spray_max = 0;
    foreach my $x (keys %spray) {
      my $row = $spray{$x};
      foreach my $y (keys %$row) {
        my $t = $row->{$y};
        $total += $t;
        $spray_max = max($spray_max, $t);
      }
    }
    print $fh "% total spray $total\n";

    my @spray_histogram = (0)x106;
    foreach my $x (sort {$a <=> $b} keys %spray) {
      my $row = $spray{$x};
      foreach my $y (sort {$a <=> $b} keys %$row) {
        my $t = $row->{$y} || next;
        $spray_histogram[$t]++;
        my $colour = $weight_to_colour->($t);

        # 4
        # 3        *
        # 2     *     *
        # 1        *
        # 0
        #    0  1  2  3  4
        my ($x1,$y1, $x2,$y2, $x3,$y3);
        if ($y % 4 == 1) {
          $x1 = $x-2;
          $x2 = $x+2;
          $x3 = $x;
          $y1 = $y-1;
          $y2 = $y-1;
          $y3 = $y+1;
        } elsif ($y % 4 == 3) {
          $x1 = $x-2;
          $x2 = $x+2;
          $x3 = $x;
          $y1 = $y+1;
          $y2 = $y+1;
          $y3 = $y-1;
        } elsif ($y % 4 == 2) {
          if ($x % 4 == 1) {
            $x1 = $x-1;
            $x2 = $x-1;
            $x3 = $x+1;
            $y1 = $y+2;
            $y2 = $y-2;
            $y3 = $y;
          } elsif ($x % 4 == 3) {
            $x1 = $x+1;
            $x2 = $x+1;
            $x3 = $x-1;
            $y1 = $y+2;
            $y2 = $y-2;
            $y3 = $y;
          } else {
            die;
          }
        } else {
          die;
        }

        print $fh "  \\fill$colour ($x1,$y1)--($x2,$y2)--($x3,$y3);\n";
      }
    }

    {
      my $path = Math::PlanePath::CCurve->new;
      print $fh "\\draw (0,0)";
      my ($n_lo, $n_hi) = $path->level_to_n_range($level);
      foreach my $n (0 .. $n_hi) {
        my ($x,$y) = $path->n_to_xy($n);
        $x *= 4;
        $y *= 4;
        print $fh "--($x,$y)";
      }
      print $fh ";\n";
    }

    print $fh "\\end{tikzpicture}\n\\end{center}\n";
    print $fh "\\pagebreak\n";

    {
      my %new;
      foreach my $x (keys %triangles) {
        my $row = $triangles{$x};
        foreach my $y (keys %$row) {
          my $t = $row->{$y};

          $new{$x}{$y} += $t;
          my ($x,$y) = xy_rotate_plus90($x,$y);
          $new{$ex+$x}{$ey+$y} += $t;
          $total += 2*$t;
        }
      }
      %triangles = %new;
      ($ex,$ey) = mul($ex,$ey);
    }

    {
      my @new_histogram;
      for (my $i = 0; $i <= $#spray_histogram; $i++) {
        $new_histogram[int($i/3)] += $spray_histogram[$i];
      }
      @spray_histogram = @new_histogram;
    }

    my $spray_total = sum(@spray_histogram);
    my $spray_total_weighted
      = sum(map {$_*$spray_histogram[$_]} 0 .. $#spray_histogram)
      / 105;
    print "  spray max $spray_max   locations $spray_total weighted $spray_total_weighted\n";

    # for (my $i = 0; $i <= $#spray_histogram; $i++) {
    #   if ($i > 0 && $i % 3 == 0) { next; }
    #   if ($spray_histogram[$i] != 0) { warn "non-zero at i=$i"; }
    # }

    my $spray_count_max = max(@spray_histogram);
    foreach my $i (0 .. $#spray_histogram) {
      my $frac = $spray_histogram[$i] / $spray_count_max;
      printf " %3d ", $i;
      my $len = ceil($frac*60);
      print '*' x $len;
      print "\n";
    }
  }

  print $fh "\\end{document}\n";
  close $fh or die;
  system("cd /tmp; pdflatex cc.tex");
  system("xpdf -z 400 /tmp/cc.pdf");
  exit 0;
}
{
  # weight in triangle

  my $path = Math::PlanePath::CCurve->new;
  require Math::PlanePath::DragonCurve;
  $path = Math::PlanePath::DragonCurve->new;
  foreach my $k (1 .. 7) {
    print "k=$k\n";
    my %xyxy_is_traversed;
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n (0 .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      $xyxy_is_traversed{$x1}{$y1}{$x2}{$y2} = 1;
       $xyxy_is_traversed{$x2}{$y2}{$x1}{$y1} = 1;  # both ways for dragon
    }
    ### %xyxy_is_traversed

    my $xyxy_to_weight = sub {
      my ($x1,$y1, $x2,$y2) = @_;
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      my $weight = 0;
      ### weight of: "$x1,$y1 to $x2,$y2  dxdy=$dx,$dy  right $rx,$ry"

      # horizontal
      if ($xyxy_is_traversed{$x1}{$y1} {$x2}{$y2}) { $weight += 2; }
      if ($xyxy_is_traversed{$x1-$dx}{$y1-$dy} {$x1}{$y1}) { $weight += 8; }
      if ($xyxy_is_traversed{$x2}{$y2} {$x2+$dx}{$y2+$dy}) { $weight += 8; }

      # horizontal, underneath
      if ($xyxy_is_traversed{$x2}{$y2} {$x1}{$y1}) { $weight += 1; }
      if ($xyxy_is_traversed{$x1}{$y1}{$x1-$dx} {$y1-$dy}) { $weight += 4; }
      if ($xyxy_is_traversed{$x2+$dx}{$y2+$dy} {$x2}{$y2}) { $weight += 4; }

      # horizontal, top 9
      if ($xyxy_is_traversed{$x2+$rx}{$y2+$ry}{$x1+$rx}{$y1+$ry}) { $weight += 24; }

      # start vertical upwards
      if ($xyxy_is_traversed{$x1}{$y1}{$x1+$rx}{$y1+$ry}) { $weight += 16; }
      if ($xyxy_is_traversed{$x1-$rx}{$y1-$ry}{$x1}{$y1}) { $weight += 2; }
      # start vertical downwards
      if ($xyxy_is_traversed{$x1+$rx}{$y1+$ry}{$x1}{$y1}) { $weight += 8; }
      if ($xyxy_is_traversed{$x1}{$y1}{$x1-$rx}{$y1-$ry}) { $weight += 1; }

      # end vertical upwards
      if ($xyxy_is_traversed{$x2}{$y2}{$x2+$rx}{$y2+$ry}) { $weight += 8; }
      if ($xyxy_is_traversed{$x2-$rx}{$y2-$ry}{$x2}{$y2}) { $weight += 1; }
      # end vertical downwards
      if ($xyxy_is_traversed{$x2+$rx}{$y2+$ry}{$x2}{$y2}) { $weight += 16; }
      if ($xyxy_is_traversed{$x2}{$y2}{$x2-$rx}{$y2-$ry}) { $weight += 2; }

      (2+8+8) + (1+4+4) + 24 + 2*(16+2+8+1) == 105 or die;
      return $weight;
    };

    my $max_weight = 0;
    my $both = 0;
    my $square = 0;
    foreach my $n (0 .. $n_hi-1) {
      my ($x1,$y1) = $path->n_to_xy($n);
      my ($x2,$y2) = $path->n_to_xy($n+1);
      my $dx = $x2 - $x1;
      my $dy = $y2 - $y1;
      my $weight = $xyxy_to_weight->($x1,$y1, $x2,$y2);
      $max_weight = max($max_weight, $weight);
      ### segment: "n=$n  weight=$weight"

      if ($k <= 14 && $weight == 105) {
         print "  n=$n has 105\n";
      }

      my $w2 = $xyxy_to_weight->($x2,$y2, $x1,$y1);
      if (($k & 1) == 1 && $weight == 105 && $w2 == 105) {
        print "  n=$n has 105 both sides\n";
        $both++;
      }

      #        wt
      #      ----->
      #     ^      |
      #  we |      v  ws
      #      <-----
      #       weight

      my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
      my $we = $xyxy_to_weight->($x2,$y2, $x2+$rx,$y2+$ry);
      my $ws = $xyxy_to_weight->($x1+$rx,$y1+$ry, $x1,$y1);
      my $wt = $xyxy_to_weight->($x2+$rx,$y2+$ry, $x1+$rx,$y1+$ry);
      if (($k & 1) == 0 && $weight == 105 && $we == 105 && $ws == 105 && $wt == 105) {
        print "  n=$n has 4*105\n";
        $square++;
      }
    }
    
    my $both_locations = $both / 2;
    my $square_locations = $square / 4;
    print "  max weight = $max_weight  both=$both pair = $both_locations,  square=$square quads = $square_locations\n";
  }
  exit 0;
}


{
  # singles-doubles graph
  require Graph::Graph6;
  Graph::Graph6::write_graph(format => "sparse6",
                             fh => \*STDOUT,
                             edge_aref => [
                                           [0,2],
                                           [0,6],
                                           [1,2],
                                           [2,9],
                                           [3,6],
                                           [4,3],
                                           [4,9],
                                           [5,10],
                                           [6,0],
                                           [6,5],
                                           [6,8],
                                           [7,4],
                                           [7,7],
                                           [8,1],
                                           [8,8],
                                           [9,7],
                                           [10,0],
                                          ]);
  exit 0;
}
{
  # right boundary states -- high to low

  #
  #                          /^
  #                         v  \
  # r0  *<=====*      =>   *    *       r1,r2
  #
  #                           /^
  #                          v==\
  # r1  *<=====*      =>    *    *      r1,r3
  #     |                  /
  #     v                 v
  #                        \
  #
  #                           /^
  #                          v==\
  # r2  *<=====*      =>    *    *      r4,r2
  #            ^                  ^
  #            |                   \
  #                                /
  #
  #                               ^^   ^^
  #                              /  \ /==\
  # r3  *<-----*<=====*    =>   *    *    *      r4,r5
  #                   ^                    \
  #                   |                     v
  #                                        /
  #
  #                               ^^   ^^
  #                              /==\ /  \
  # r4  *<=====*<-----*    =>   *    *    *    r6,r3
  #     |                      /
  #     v                     v
  #
  #                         *
  #                          \
  #     ^                     ^/^
  #     |                    /v  \
  # r5  *<=====*            *     *     r7,non
  #            ^                   ^
  #            |      =>            \
  #
  #                              *
  #                             /
  #            |              /^\
  #            v             v  \v
  # r6  *<=====*            *    *      non,r7
  #     |                  /
  #     v             =>  v
  #
  #                                     ^^   ^^   ^^
  #                                    /  \ /==\ /  \
  # r7  *<-----*<=====*<-----*   =>   *    *    *    *   r6,r5
  #

  BEGIN {
    @right_boundary_table = ([1,2],  # r0
                             [1,3],
                             [4,2],  # r2
                             [4,5],
                             [6,3],  # r4
                             [7,8],
                             [8,7],  # r6
                             [6,5],
                             [8,8],  # s8=non
                            );
  }

  require MyFSM;
  my $table = {};
  foreach my $digit (0,1) {
    foreach my $state (0 .. $#right_boundary_table) {
      $table->{$state}->{$digit} = $right_boundary_table[$state][$digit];
    }
  }
  my $fsm = MyFSM->new
    (table => $table,
     accepting => { 8 => 'non' },
     initial   => 0,
    );


  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  $fsm->show;
  $fsm->view;

  $fsm = $fsm->reverse;
  $fsm->simplify;

  # $fsm->rename_state('0,4,2,6,0,9,8,10,10,4,10','m1');
  # $fsm->rename_state('1,3,1,3,5,7,5,10,3,10,10','m2');
  #
  # $fsm->rename_state('0,0,0,8,0,4,10,10,10,0,10','m3');
  # $fsm->rename_state('0,8,0,8,4,10,4,10,8,10,10','m4');
  # $fsm->rename_state('0,4,0,4,0,10,8,10,10,4,10','m5');
  #
  # $fsm->rename_state('0,0,0,10,0,0,10,10,10,0,10','m6');
  # $fsm->rename_state('0,0,0,0,10,10,10,10,0,10,10','m7');
  #
  # $fsm->rename_state('0,10,0,10,0,10,0,10,10,10,10','m8');
  # $fsm->rename_state('0,0,0,0,0,10,10,10,10,0,10','m9');

  print "------\n";
  print "reversed\n";
  $fsm->show;
  $fsm->view;
  print $fsm->as_graph6;
  print $fsm->as_sparse6;
  exit 0;

  sub right_boundary_state_HtoL {
    my ($n,$level) = @_;
    ### right_boundary_state_HtoL(): "$n in level $level"
    my $state = 0;
    my @bits = bit_split_lowtohigh($n);
    if (@bits < $level) { push @bits, 0 }
    @bits = reverse @bits;  # high to low
    ### bits: join(' ',@bits)
    foreach my $bit (@bits) {
      ### at: $state
      $state = $right_boundary_table[$state][$bit];
    }
    ### final: $state
    return $state;
  }
  sub is_right_boundary_HtoL {
    my ($n,$level) = @_;
    return right_boundary_state_HtoL($n,$level) != 8;
  }
}
{
  # centroid N

  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $x_total = 0;
    my $y_total = 0;
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      $x_total += 2*$x + $dx;
      $y_total += 2*$y + $dy;
      $count++;
    }

    my ($Gx,$Gy) = G_xy($k);
    my $gx = $x_total / $count / 2;
    my $gy = $y_total / $count / 2;
    my @n_list = $path->xy_to_n_list($gx,$gy);
    my $n_list = join(', ',@n_list);
    print "k=$k  $gx,$gy  $Gx,$Gy  N=$n_list\n";

    if ($k % 2 == 0) {
    }
    push @values, $n_list[0];
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}

{
  # number of double-visited segments across next level
  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 16) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $n_next_hi = 2*$n_hi;
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $n_other = $path->xyxy_to_n($x+$dx,$y+$dy, $x,$y) // next;
      next unless $n_other >= $n_hi && $n_other < $n_next_hi;
      $count++;
    }
    print "k=$k  $count\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}




{
  # double seg rotations
  my $path = Math::PlanePath::CCurve->new;
  my $n1 =  0b111_0000;
  my $n2 = 0b1000_1111;
  $,=', ';
  print $path->n_to_xy($n1),$path->n_to_dxdy($n1),"\n";
  print $path->n_to_xy($n2),$path->n_to_dxdy($n2),"\n";
  print "delta ",$n2-$n1,"\n";
  exit 0;
}

{
  # number of pair configurations
  my $path = Math::PlanePath::CCurve->new;

  print "     ";
  foreach my $i (1 .. 11) { printf " %4d", $i; }
  print "\n";

  my @pair = (undef,
              # *===>*--->*  or  *--->*===>*
              [[1,0, 2,0],   [-1,0, 0,0]],   # p1

              #              *--->*
              #                    
              # *===>*       *===>*
              #          or
              # *--->*       
              [[0,-1, 1,-1], [0,1, 1,1]],  # p2

              #                  *--->*
              #
              # *===>*      or        *===>*
              #               
              #      *--->*
              #
              #                       *--->*
              #
              #      *===>*  or  *===>*
              #               
              # *--->*
              [[1,-1, 2,-1],   [-1,1, 0,1],   # p3
               [-1,-1, 0,-1], [1,1, 2,1]],

              # *===>*  
              #         
              # *<---*  
              [[1,-1, 0,-1]],  # p4

              #      *===>*
              #            
              # *<---*     
              #
              #      *===>*
              #            
              #           *<---*     
              [ [0,-1, -1,-1], ,   # p5
                [2,-1, 1,-1] ],

              # *===>*
              # *<---*
              [ [1,0, 0,0]],  # p6

              # *===>*     *===>*
              # ^               |
              # |      or       v    are already mirror images
              # *               *
              #
              # *===>*       *===>*
              #      |       ^
              #      v  or   |
              #      *       *
              [[0,-1, 0,0],   [1,0, 1,-1] ],    # p7

              # *    *===>*        *===>*
              # ^
              # |            or
              # *                       *
              #                         |
              #                         v
              #                         *
              #
              # *===>*    *         *===>*
              #           |    
              #           v  or
              #           *         *
              #                     ^
              #                     |
              #                     *
              #
              [[-1,-1, -1,0],   [1,-1, 1,-2],  # p8
               [2,0, 2,-1], [0,-2, 0,-1]],

              # *===>*     *
              # |          ^
              # v      or  |
              # *          *===>*
              #
              # *===>*            *
              #      ^            |
              #      |   or       v
              #      *       *===>*
              [[0,0, 0,-1],   [0,0, 0,1],    # p9
               [1,-1, 1,0],   [1,1, 1,0] ],

              #                    *===>*
              #                    
              #                    
              # *                  *
              # ^                  |
              # |            or    v    
              # *    *===>*        *

              #                       *===>*
              #                    
              #                    
              #            *               *    
              #            |               ^
              #            v    or         |    
              # *===>*     *               *

              [[-1,0, -1,1],  [0,-1, 0,-2],  # p10
               [2,1, 2,0], [1,-2, 1,-1]],

              # *                  *
              # |                  ^
              # v       or         |      mirror image same
              # *===>*        *===>*
              #
              [[0,1, 0,0],  [1,0, 1,1]],   # p11
             );

  my $verbose = 1;
  my @values;
  foreach my $k (0 .. 15) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $n_next_hi = 2*$n_hi;
    my @count = ('', (0)x11);
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $rot = dxdy_to_dir4($dx,$dy);

      foreach my $p (1 .. $#pair) {
        my $aref = $pair[$p];
        foreach my $e (0 .. $#$aref) {
          my $elem = $aref->[$e];
          my ($dx1,$dy1, $dx2,$dy2) = @$elem;

          foreach (1 .. $rot) {
            ($dx1,$dy1) = xy_rotate_plus90($dx1,$dy1);
            ($dx2,$dy2) = xy_rotate_plus90($dx2,$dy2);
          }
          my $x1 = $x + $dx1;
          my $y1 = $y + $dy1;
          my $x2 = $x + $dx2;
          my $y2 = $y + $dy2;
          my $n_other = $path->xyxy_to_n($x1,$y1, $x2,$y2) // next;
          # next unless $n < $n_other && $n_other < $n_hi &&;
          next unless $n_other >= $n_hi && $n_other < $n_next_hi;
          if ($verbose && ($k <= 3
                          # || $p==7
                          )) {
            my ($dx,$dy) = $path->n_to_dxdy($n);
            print "  p$p  n=$n at $x,$y   $dx,$dy other n=$n_other relative($e) $dx1,$dy1, $dx2,$dy2\n";
          }
          $count[$p]++;
          $count[12]++;
        }
      }
    }

    printf "k=%2d ", $k;
    foreach my $i (1 .. $#count) { printf " %4d", $count[$i]; }
    print "\n";
    push @values, $count[6];
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}



{
  # number of double-visited segments
  my $path = Math::PlanePath::CCurve->new;
  my @values;
  foreach my $k (0 .. 12) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    my $count = 0;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $n_other = $path->xyxy_to_n($x+$dx,$y+$dy, $x,$y) // next;
      next if $n_other >= $n_hi;
      next if $n_other < $n;
      $count++;
    }
    print "k=$k  $count\n";
    push @values, $count;
  }
  require Math::OEIS::Grep;
  Math::OEIS::Grep->search(array => \@values, verbose=>1);
  exit 0;
}


{
  my $path = Math::PlanePath::CCurve->new;
  foreach my $level (0 .. 8) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($level);
    foreach my $n ($n_lo .. $n_hi-1) {

      my @bits = bit_split_lowtohigh($n);
      while (@bits < $level) { push @bits, 0 }
      @bits = reverse @bits;  # high to low

      # @bits = bits_HtoL_strip_high_run(@bits);
      @bits = bits_HtoL_strip_high_duplicates(@bits);
      my @trans = bits_to_transitions(@bits);
      my @t2 = bits_to_transitions(@trans);

      my $want = is_right_boundary_HtoL($n,$level) ? 1 : 0;
      my $n2 = sprintf '%0*b', $level,$n;
      my $bits = join('',@bits);
      my $trans = join('',@trans);
      my $t2 = join('',@t2);
      my $count = count_zeros(@t2);
      $count = count_nonzeros(@t2);
      my $got = any_even_run_ones_between_zeros(@trans) ? 0 : 1;
      my $diff = ($got == $want ? '' : ' ***');
      print "k=$level n=$n [$n2 $bits]  transitions=$trans,$t2  got=$got want=$want$diff\n";
    }
  }
  exit 0;

  sub bits_to_transitions {
    my @bits = @_;
    foreach my $i (0 .. $#bits-1) {
      $bits[$i] = ($bits[$i] == $bits[$i+1] ? 0 : 1);
    }
    pop @bits;
    return @bits;
  }
  sub count_zeros {
    my $count = 0;
    foreach my $n (@_) { $count += ($n == 0); }
    return $count;
  }
  sub count_nonzeros {
    my $count = 0;
    foreach my $n (@_) { $count += ($n != 0); }
    return $count;
  }
  sub any_even_run_ones_between_zeros {
    my @bits = @_;
    while (@bits && $bits[0] == 1) {
      shift @bits;
    }
    while (@bits) {
      shift @bits;  # zero
      my $ones = 0;
      while (@bits && $bits[0] == 1) {
        shift @bits;
        $ones++;
      }
      if (! @bits) {
        return 0;
      }
      if ($ones % 2 == 0) {
        return 1;
      }
    }
    return 0;
  }

  sub bits_HtoL_strip_high_duplicates {
    my @bits = @_;
    while (@bits >= 2 && $bits[0] == $bits[1]) {
      shift @bits;
    }
    return @bits;
  }
}

{
  my $bad = 0;
  my $path = Math::PlanePath::CCurve->new;
  foreach my $k (0 .. 14) {
    my ($n_lo, $n_hi) = $path->level_to_n_range($k);
    foreach my $n ($n_lo .. $n_hi-1) {
      my $is_htol = is_right_boundary_HtoL($n,$k) ? 1 : 0;
      my $is_bits = is_right_boundary_bits($n,$k) ? 1 : 0;
      my $is_path = is_right_boundary_path($path,$n,$n_hi) ? 1 : 0;
      if ($is_htol != $is_bits || $is_htol != $is_path) {
        my $n2 = sprintf '%0*b', $k,$n;
        my @pairs = n_to_pairs($n,$k);
        my $pairs = join('',@pairs);
        my $state = right_boundary_state_HtoL($n,$k);
        print "k=$k n=$n [$n2]  path=$is_path  HtoL=$is_htol (state=$state)  bits=$is_bits pairs=$pairs\n";
        die if $bad++ > 10;
      }
    }
  }
  exit 0;

  sub is_right_boundary_bits {
    my ($n,$level) = @_;
    my @pairs = n_to_pairs($n,$level);
    foreach my $i (0 .. $#pairs-1) {
      if ($pairs[$i] == $pairs[$i+1]) {
        return 0;
      }
    }
    return 1;
  }
  sub n_to_pairs {
    my ($n,$level) = @_;
    my @bits = bit_split_lowtohigh($n);
    while (@bits < $level) { push @bits, 0 }
    @bits = reverse @bits;
    @bits = bits_HtoL_strip_high_run(@bits);

    # while (@bits >= 3 && $bits[0] == $bits[1] && $bits[1] == $bits[2]) {
    #   shift @bits;
    # }

    my @pairs;
    foreach my $i (0 .. $#bits-1) {
      if ($bits[$i] == $bits[$i+1]) {
        push @pairs, $bits[$i];
      }
    }
    return @pairs;
  }

  sub bits_HtoL_strip_high_run {
    my @bits = @_;
    if (@bits >= 1) {
      my $high = shift @bits;
      while (@bits && $bits[0] == $high) {
        shift @bits;
      }
    }
    return @bits;
  }

  sub is_right_boundary_path {
    my ($path, $n, $n_hi) = @_;
    ### is_right_boundary_path(): "$n in $n_hi"
    while ($n_hi > 8) {
      ### at: "$n in $n_hi"
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my $n_other = $path->xyxy_to_n($x+$dx,$y+$dy, $x,$y);
      ### $n_other
      if (defined $n_other && $n_other < $n_hi) {
        ### no, not boundary ...
        return 0;
      }
      $n_hi /= 2;
      $n >>= 1;
    }
    ### yes, boundary ...
    return 1;
  }
}



{
  # turn runs
  # max turn right 3 at N=15
  require Math::NumSeq::PlanePathTurn;
  my $seq = Math::NumSeq::PlanePathTurn->new(planepath=>'CCurve',
                                             turn_type => 'Turn4');
  my ($run_i,$run_value) = $seq->next;
  my $run_length = 1;
  my @max_run = (0,0,0,0);
  for (1 .. 1_000_000) {
    my ($i,$value) = $seq->next;
    if ($value == $run_value) {
      $run_length++;
    } else {
      if ($run_length > $max_run[$run_value]) {
        $max_run[$run_value] = $run_length;
        print "i=$run_i turn=$run_value  length=$run_length\n";
      }
      $run_value = $value;
      $run_i = $i;
      $run_length = 1;
    }
  }
  exit 0;
}




#------------------------------------------------------------------------------
sub dxdy_to_dir4 {
  my ($dx, $dy) = @_;
  if ($dx > 0) { return 0; }  # east
  if ($dx < 0) { return 2; }  # west
  if ($dy > 0) { return 1; }  # north
  if ($dy < 0) { return 3; }  # south
}

sub dir4_are_right_angles {
  my ($d1,$d2) = @_;
  return ($d1 & 1) != ($d2 & 1);
}
