#!/usr/bin/perl -w

# Copyright 2014, 2015, 2017, 2018, 2022 Kevin Ryde
#
# This file is free software; you can redistribute it and/or modify it
# under the terms of the GNU General Public License as published by the Free
# Software Foundation; either version 3, or (at your option) any later
# version.
#
# This file is distributed in the hope that it will be useful, but
# WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
# or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
# for more details.
#
# You should have received a copy of the GNU General Public License along
# with this file.  If not, see <http://www.gnu.org/licenses/>.


use 5.004;
use strict;
use FindBin;
use List::Util 'min','max';
use Math::BigRat;
use Math::PlanePath::CCurve 117; # v.117 for level_to_n_range()
use Math::Geometry::Planar ();

use FindBin;
use lib $FindBin::Bin;
BEGIN { unshift @INC, $FindBin::Bin }
use MyCCurve 'xy_rotate_plus90','xy_rotate_minus90',
  'xydxdy_reverse','xydxdy_triangle_neighbours',
  'xydxdy_triangle_to_gpc';
use MyPrintwrap ':all';
use MyPlanar;
$| = 1;

# uncomment this to run the ### lines
# use Smart::Comments;


# ENHANCE-ME: This program started as a print 4 things in one go, but think
# prefer separate runs for ease of checking the output pasted into
# c-curve.tex.  Maybe would re-arrange a bit so generate just the requested
# level.

my $max_level = 16;
my $want_level = -1;      # printout, or -1 for sample values
if (@ARGV) { $want_level = shift @ARGV; }

# $aref is an arrayref of triangles in the form of arrayrefs [x,y,dx,dy]
# Return a Math::Geometry::Planar::GPC which is their areas, unioned together.
#
sub xydxdy_triangles_to_gpc {
  my ($aref) = @_;
  my $gpc = xydxdy_triangle_to_gpc(@{$aref->[0]});
  foreach my $i (1 .. $#$aref) {
    my $gpc2 = xydxdy_triangle_to_gpc(@{$aref->[$i]});
    $gpc = Math::Geometry::Planar::GpcClip('UNION', $gpc, $gpc2);
  }
  return $gpc;
}

# $aref is an arrayref of triangles in the form of arrayrefs [x,y,dx,dy]
# Return an arrayref of points [x,y] which is the polygon around their union.
# The triangles must be a contiguous polygon, meaning everything reachable
# across sides in common.
#
sub xydxdy_triangles_to_polygon {
  my ($aref) = @_;
  my @list = MyPlanar::gpc_to_points_list(xydxdy_triangles_to_gpc($aref));
  @list==1 or die;
  return $list[0];
}

# Return ($cx,$cy) lower left corner of the unit square on the right of the
# given segment $x,$y,$dx,$dy.
sub xydxdy_to_lower_left {
  my ($x,$y, $dx,$dy) = @_;
  my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
  return (min($x, $x+$dx+$rx), min($y, $y+$dy+$ry));
}

# $list is an arrayref of numbers.
# For all runs of consecutive equal numbers, collapse to just one of that
# run by splicing out the others.  The array is treated as circular, so the
# first and last elements are consecutive.
#
sub list_collapse {
  my ($list) = @_;
  my $i = 0;
  while ($i <= $#$list) {
    if ($list->[$i] == $list->[$i-1]) {
      splice @$list, $i, 1;
    } else {
      $i++;
    }
  }
}

sub xydxdy_segments_bbox {
  my ($aref) = @_;
  my @x = map {my ($x,$y,$dx,$dy) = @$_;
               ($x, $x+$dx)} @$aref;
  my @y = map {my ($x,$y,$dx,$dy) = @$_;
               ($y, $y+$dy)} @$aref;
  return (min(@x),min(@y), max(@x),max(@y));
}


my $path = Math::PlanePath::CCurve->new;
my @Trigons_samples;
my @VTouch_samples;
my @V4_samples;
my @QTouch_samples;
my @QI4_samples;
my $seen_multi_touch = 1;
my $seen_self_touch;
foreach my $k (#0..12,
               #0 .. 11,
               #11,
               0 .. $max_level,
              ) {
  my ($n_lo, $n_hi) = $path->level_to_n_range($k);
  my @n_to_trigon;
  my %xydxdy_to_trigon;
  my @trigon_to_n_list;
  my $this_multi_touch;
  my $this_self_touch;

  my %points;

  foreach my $n ($n_lo .. $n_hi-1) {
    next if defined $n_to_trigon[$n];

    my ($x,$y) = $path->n_to_xy($n);
    my ($dx,$dy) = $path->n_to_dxdy($n);

    # this triangle is a new polygon piece, extend out its neighbours
    my $trigon = scalar(@trigon_to_n_list);
    push @{$trigon_to_n_list[$trigon]}, $n;
    $n_to_trigon[$n] = $trigon;
    $xydxdy_to_trigon{"$x,$y,$dx,$dy"} = $trigon;

    my @pending = (xydxdy_triangle_neighbours($x,$y, $dx,$dy));
    while (my $seg = shift @pending) {
      my ($x,$y, $dx,$dy) = @$seg;
      my $n = $path->xyxy_to_n ($x,$y, $x+$dx,$y+$dy) // next;
      $n < $n_hi or next;
      if (defined(my $got = $n_to_trigon[$n])) {
        unless ($got == $trigon) {
          die "oops, k=$k trigon=$trigon n=$n but n_to_trigon[n]=$n_to_trigon[$n]";
        }
        next;
      }
      push @{$trigon_to_n_list[$trigon]}, $n;
      $n_to_trigon[$n] = $trigon;
      $xydxdy_to_trigon{"$x,$y,$dx,$dy"} = $trigon;

      push @pending, xydxdy_triangle_neighbours($x,$y, $dx,$dy);
    }
  }

  #         ^^
  #       <-----
  #       ------>
  #         vv
  #
  my @trigon_touch;
  my $process_list = sub {
    my ($list) = @_;
    ### process_list: $list
    return if @$list <= 1;

    {
      # self touches, and unduplicate @$list
      my %seen;
      my $i = 0;
      while ($i <= $#$list) {
        my $trigon = $list->[$i];
        if ($seen{$trigon}++) {
          ### duplicate: $trigon
          splice @$list, $i, 1;  # remove

          $seen{$trigon} <= 2 or die;
          unless ($seen_self_touch) {
            if ($seen{$trigon} == 2) {  # second occurrence count self-touch
              # Forget what this print was for.
              # print "% k=$k self touch $trigon\n";
              $this_self_touch = 1;
              $trigon_touch[$trigon]->{$trigon}++;
            }
          }
        } else {
          $i++;
        }
      }
    }

    # cross-touches, with multiplicity
    foreach my $i (0 .. $#$list) {
      foreach my $j ($i+1 .. $#$list) {
        my $from_trigon = $list->[$i];
        my $to_trigon   = $list->[$j];
        $from_trigon != $to_trigon or die;
        $trigon_touch[$from_trigon]->{$to_trigon}++;
        $trigon_touch[$to_trigon]->{$from_trigon}++;
      }
    }
  };

  my @VTouch;
  {
    my %seen;
    foreach my $n ($n_lo .. $n_hi) {
      my ($x,$y) = $path->n_to_xy($n);
      next if $seen{"$x,$y"}++;
      my $dx = 1;
      my $dy = 0;
      my @list;
      foreach (1..4) {
        foreach (0,1) {
          push @list, $xydxdy_to_trigon{"$x,$y,$dx,$dy"} // -1;
          ($x,$y, $dx,$dy) = xydxdy_reverse($x,$y, $dx,$dy);
        }
        ($dx,$dy) = xy_rotate_plus90($dx,$dy);
      }
      list_collapse(\@list);
      @list = grep {$_>=0} @list;
      my $t = scalar(@list);
      unless ($t >= 0 && $t <= 3) {
        die;
      }
      $VTouch[$t]++;
      $process_list->(\@list);
    }
  }

  #       ------>
  #         vv   |
  #             <|
  #              v
  my @QTouch;
  {
    my %seen;
    foreach my $n ($n_lo .. $n_hi-1) {
      my ($x,$y) = $path->n_to_xy($n);
      my ($dx,$dy) = $path->n_to_dxdy($n);
      my ($cx,$cy) = xydxdy_to_lower_left($x,$y, $dx,$dy);
      next if $seen{"$cx,$cy"}++;
      my @list;
      foreach (1..4) {
        push @list, $xydxdy_to_trigon{"$x,$y,$dx,$dy"} // -1;
        $x += $dx; $y += $dy;
        ($dx,$dy) = xy_rotate_minus90($dx,$dy);
      }
      ### @list
      list_collapse(\@list);
      @list = grep {$_>=0} @list;
      my $t = scalar(@list);
      unless ($t >= 0 && $t <= 2) {
        print "k=$k n=$n got t=$t list=",join(',',@list),"\n";
        die;
      }
      $QTouch[$t]++;
      $process_list->(\@list);
    }
  }

  my %multi_trigons;
  foreach my $from_trigon (0 .. $#trigon_touch) {
    my $href = $trigon_touch[$from_trigon] // next;
    foreach my $to_trigon (sort {$a<=>$b} keys %$href) {
      my $count = $href->{$to_trigon};
      if ($count > $seen_multi_touch) {
        my $n = $trigon_to_n_list[$from_trigon]->[0];
        my ($x,$y) = $path->n_to_xy($n);
        ($x,$y) = MyCCurve::transform_to_horizontal($x,$y,$k);
        # print "% k=$k multi touch $from_trigon to $to_trigon count $count near $x,$y\n";
        $this_multi_touch = $count;
        $multi_trigons{$from_trigon} = 1;
        $multi_trigons{$to_trigon} = 1;
      }
    }
  }

  if ($k == $want_level) {
    %multi_trigons or die "oops, no multi_trigons";
    print "  % generated by tools/$FindBin::Script $want_level\n";
    print "  % $this_multi_touch touches\n";
    print "  \\newcommand{\\MyLevel}{$k}\n";
    my $i = 1;
    foreach my $trigon (sort {$a<=>$b} keys %multi_trigons) {
      my $n_list = $trigon_to_n_list[$trigon];
      my @xydxdy_triangles
        = map {my $n = $_;
               my ($x,$y) = $path->n_to_xy($n);
               my ($dx,$dy) = $path->n_to_dxdy($n);
               ($x,$y) = MyCCurve::transform_to_horizontal($x,$y, $k);
               ($dx,$dy) = MyCCurve::transform_to_horizontal($dx,$dy, $k);
               [$x,$y, $dx,$dy]
             } @$n_list;
      next if $k==14 && $xydxdy_triangles[0]->[0] < -64;
      next if $k==16 && $xydxdy_triangles[0]->[0] < -128;

      my $points = xydxdy_triangles_to_polygon(\@xydxdy_triangles);
      $Printwrap = 2;
      printwrap("  \\fill ");
      $Printwrap += 2;
      my $join = '';
      foreach my $p (@$points) {
        my ($x,$y) = @$p;
        printwrap("$join($x,$y)");
        $join = '--';
      }
      printwrap("--cycle;\n");

      if ($k >= 14) {
        # untransformed
        @xydxdy_triangles
          = map {my $n = $_;
                 my ($x,$y) = $path->n_to_xy($n);
                 my ($dx,$dy) = $path->n_to_dxdy($n);
                 [$x,$y, $dx,$dy]
               } @$n_list;
        my ($x1,$y1,$x2,$y2) = xydxdy_segments_bbox(\@xydxdy_triangles);
        my $count = scalar(@xydxdy_triangles);
        print "% GP-DEFINE  gen_Touches_${k}_${i}_count = $count;\n";
        print "% GP-DEFINE  gen_Touches_${k}_${i}_bbox = [$x1+I*$y1,$x2+I*$y2];\n";
        $i++;
      }
    }
    print "  % end generated\n";
    exit 0;
  }

  push @Trigons_samples, scalar(@trigon_to_n_list);
  foreach my $t (1 .. 3) {
    push @{$VTouch_samples[$t]}, $VTouch[$t];
  }
  foreach my $t (1 .. 2) {
    push @{$QTouch_samples[$t]}, $QTouch[$t];
  }
  push @QI4_samples, $QTouch[0];
  push @V4_samples, $VTouch[0];
  if ($this_multi_touch) {
    $seen_multi_touch = $this_multi_touch;
  }
  $seen_self_touch ||= $this_self_touch;
}

print "% generated by tools/$FindBin::Script $want_level\n";
foreach my $elem (['Trigons', \@Trigons_samples],
                  ['VTouch1', $VTouch_samples[1]],
                  ['VTouch2', $VTouch_samples[2]],
                  ['VTouch3', $VTouch_samples[3]],
                  ['V4', \@V4_samples],
                  ['QTouch1', $QTouch_samples[1]],
                  ['QTouch2', $QTouch_samples[2]],
                  ['QI4', \@QI4_samples],
                 ) {
  my ($name,$aref) = @$elem;
  print "% GP-DEFINE  gen_Touches_${name}_samples = [";
  my $join = '';
  foreach my $k (0 .. $#$aref) {
    print $join,($aref->[$k]//0);
    $join = ',';
  }
  print "];\n";
}
print "% end generated\n";
exit 0;









# sub xydxdy_starts {
#   my ($x,$y, $dx,$dy) = @_;
#   my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
#   my ($lx,$ly) = (-$rx,-$ry);
#   my @starts = ([$x,$y, -$dx,-$dy],
#                 [$x,$y, $rx,$ry],
#                 [$x,$y, $lx,$ly]);
#   return ([xydxdy_reverse($x,$y,$dx,$dy)],
#           @starts,
#           map {[xydxdy_reverse(@$_)]} @starts);
# }
# sub xydxdy_ends {
#   my ($x,$y, $dx,$dy) = @_;
#   my $x2 = $x + $dx;
#   my $y2 = $y + $dy;
#   my ($rx,$ry) = xy_rotate_minus90($dx,$dy);
#   my ($lx,$ly) = (-$rx,-$ry);
#   my @ends = ([$x2,$y2, $dx,$dy],
#               [$x2,$y2, $rx,$ry],
#               [$x2,$y2, $lx,$ly]);
#   return ([xydxdy_reverse($x,$y,$dx,$dy)],
#           @ends,
#           map {[xydxdy_reverse(@$_)]} @ends);
# }
# sub xydxdy_tops {
#   my ($x,$y, $dx,$dy) = @_;
#   my @ret;
#   foreach (1 .. 3) {
#     my $x = $x + $dx;
#     my $y = $y + $dy;
#     my ($dx,$dy) = xy_rotate_minus90($dx,$dy);
#     push @ret, [$x,$y,$dx,$dy];
#   }
# }
